/*                     __                                               *\
**     ________ ___   / /  ___      __ ____  Scala.js Test Suite        **
**    / __/ __// _ | / /  / _ | __ / // __/  (c) 2013, LAMP/EPFL        **
**  __\ \/ /__/ __ |/ /__/ __ |/_// /_\ \    http://scala-js.org/       **
** /____/\___/_/ |_/____/_/ | |__/ /____/                               **
**                          |/____/                                     **
\*                                                                      */
package org.scalajs.testsuite.javalib.lang

import org.scalajs.jasminetest.JasmineTest
import scala.scalajs.js

object CharacterTest extends JasmineTest {

  describe("java.lang.Character") {

    it("should provide `isISOControl`") {
      val isoControlChars = (('\u0000' to '\u001F') ++
          ('\u007F' to '\u009F')).map(_.toInt).toSet
      isoControlChars foreach { c =>
        expect(Character.isISOControl(c)).toEqual(true)
      }

      val randomInts = List.fill(100)(scala.util.Random.nextInt)
      ((-1000 to 1000) ++ randomInts).filterNot(isoControlChars) foreach { c =>
        expect(Character.isISOControl(c)).toEqual(false)
      }
    }

    it("should provide `digit`") {
      expect(Character.digit('a', 16)).toEqual(10)
      expect(Character.digit('}', 5)).toEqual(-1)
      expect(Character.digit('1', 50)).toEqual(-1)
      expect(Character.digit('1', 36)).toEqual(1)
      expect(Character.digit('Z', 36)).toEqual(35)
      expect(Character.digit('\uFF22', 20)).toEqual(11)
    }

    it("should provide `forDigit`") {
      /* Ported from
       * https://github.com/gwtproject/gwt/blob/master/user/test/com/google/gwt/emultest/java/lang/CharacterTest.java
       */
      for (i <- 0 until 36) {
        expect(Character.digit(Character.forDigit(i, 36), 36)).toEqual(i)
      }
      expect(Character.forDigit(9, 10) == '9').toBeTruthy
    }

    it("should provide `toChars`") {
      expect(Character.toChars(0x61) sameElements Array('a')).toBeTruthy
      expect(Character.toChars(0x10000) sameElements Array('\uD800', '\uDC00')).toBeTruthy
      expect(Character.toChars(0x10001) sameElements Array('\uD800', '\uDC01')).toBeTruthy
      expect(Character.toChars(0x10401) sameElements Array('\uD801', '\uDC01')).toBeTruthy
      expect(Character.toChars(0x10FFFF) sameElements Array('\uDBFF', '\uDFFF')).toBeTruthy

      expect(() => Character.toChars(Integer.MAX_VALUE)).toThrow
    }

    it("should provide isDigit") {
      expect(Character.isDigit('a')).toBeFalsy
      expect(Character.isDigit('0')).toBeTruthy
      expect(Character.isDigit('5')).toBeTruthy
      expect(Character.isDigit('9')).toBeTruthy
      expect(Character.isDigit('z')).toBeFalsy
      expect(Character.isDigit(' ')).toBeFalsy

      // 100 randomly chosen characters that produce true
      expect(Character.isDigit('\u0661')).toBeTruthy
      expect(Character.isDigit('\u0663')).toBeTruthy
      expect(Character.isDigit('\u0664')).toBeTruthy
      expect(Character.isDigit('\u0665')).toBeTruthy
      expect(Character.isDigit('\u0669')).toBeTruthy
      expect(Character.isDigit('\u06F1')).toBeTruthy
      expect(Character.isDigit('\u06F3')).toBeTruthy
      expect(Character.isDigit('\u06F5')).toBeTruthy
      expect(Character.isDigit('\u07C3')).toBeTruthy
      expect(Character.isDigit('\u07C7')).toBeTruthy
      expect(Character.isDigit('\u0967')).toBeTruthy
      expect(Character.isDigit('\u096C')).toBeTruthy
      expect(Character.isDigit('\u096F')).toBeTruthy
      expect(Character.isDigit('\u09E6')).toBeTruthy
      expect(Character.isDigit('\u09E8')).toBeTruthy
      expect(Character.isDigit('\u09EA')).toBeTruthy
      expect(Character.isDigit('\u09ED')).toBeTruthy
      expect(Character.isDigit('\u0A66')).toBeTruthy
      expect(Character.isDigit('\u0A69')).toBeTruthy
      expect(Character.isDigit('\u0A6D')).toBeTruthy
      expect(Character.isDigit('\u0BE9')).toBeTruthy
      expect(Character.isDigit('\u0C67')).toBeTruthy
      expect(Character.isDigit('\u0C6A')).toBeTruthy
      expect(Character.isDigit('\u0CE6')).toBeTruthy
      expect(Character.isDigit('\u0CE9')).toBeTruthy
      expect(Character.isDigit('\u0CEC')).toBeTruthy
      expect(Character.isDigit('\u0CED')).toBeTruthy
      expect(Character.isDigit('\u0CEF')).toBeTruthy
      expect(Character.isDigit('\u0D6B')).toBeTruthy
      expect(Character.isDigit('\u0D6C')).toBeTruthy
      expect(Character.isDigit('\u0E51')).toBeTruthy
      expect(Character.isDigit('\u0E53')).toBeTruthy
      expect(Character.isDigit('\u0E54')).toBeTruthy
      expect(Character.isDigit('\u0ED3')).toBeTruthy
      expect(Character.isDigit('\u0ED6')).toBeTruthy
      expect(Character.isDigit('\u0F20')).toBeTruthy
      expect(Character.isDigit('\u0F21')).toBeTruthy
      expect(Character.isDigit('\u1040')).toBeTruthy
      expect(Character.isDigit('\u1048')).toBeTruthy
      expect(Character.isDigit('\u1091')).toBeTruthy
      expect(Character.isDigit('\u1094')).toBeTruthy
      expect(Character.isDigit('\u17E0')).toBeTruthy
      expect(Character.isDigit('\u1811')).toBeTruthy
      expect(Character.isDigit('\u1814')).toBeTruthy
      expect(Character.isDigit('\u1815')).toBeTruthy
      expect(Character.isDigit('\u1946')).toBeTruthy
      expect(Character.isDigit('\u1947')).toBeTruthy
      expect(Character.isDigit('\u1948')).toBeTruthy
      expect(Character.isDigit('\u194A')).toBeTruthy
      expect(Character.isDigit('\u194C')).toBeTruthy
      expect(Character.isDigit('\u19D4')).toBeTruthy
      expect(Character.isDigit('\u19D5')).toBeTruthy
      expect(Character.isDigit('\u19D6')).toBeTruthy
      expect(Character.isDigit('\u19D9')).toBeTruthy
      expect(Character.isDigit('\u1A81')).toBeTruthy
      expect(Character.isDigit('\u1A84')).toBeTruthy
      expect(Character.isDigit('\u1A86')).toBeTruthy
      expect(Character.isDigit('\u1A87')).toBeTruthy
      expect(Character.isDigit('\u1A91')).toBeTruthy
      expect(Character.isDigit('\u1A92')).toBeTruthy
      expect(Character.isDigit('\u1A96')).toBeTruthy
      expect(Character.isDigit('\u1A98')).toBeTruthy
      expect(Character.isDigit('\u1B56')).toBeTruthy
      expect(Character.isDigit('\u1B58')).toBeTruthy
      expect(Character.isDigit('\u1BB0')).toBeTruthy
      expect(Character.isDigit('\u1BB2')).toBeTruthy
      expect(Character.isDigit('\u1C43')).toBeTruthy
      expect(Character.isDigit('\u1C51')).toBeTruthy
      expect(Character.isDigit('\uA622')).toBeTruthy
      expect(Character.isDigit('\uA624')).toBeTruthy
      expect(Character.isDigit('\uA8D1')).toBeTruthy
      expect(Character.isDigit('\uA8D6')).toBeTruthy
      expect(Character.isDigit('\uA8D8')).toBeTruthy
      expect(Character.isDigit('\uA9D2')).toBeTruthy
      expect(Character.isDigit('\uA9D8')).toBeTruthy
      expect(Character.isDigit('\uAA50')).toBeTruthy
      expect(Character.isDigit('\uAA52')).toBeTruthy
      expect(Character.isDigit('\uAA56')).toBeTruthy
      expect(Character.isDigit('\uABF5')).toBeTruthy
      expect(Character.isDigit('\uFF12')).toBeTruthy
      expect(Character.isDigit('\uFF14')).toBeTruthy
      expect(Character.isDigit('\u0033')).toBeTruthy

      // 100 randomly chosen characters that produce false
      expect(Character.isDigit('\u04A3')).toBeFalsy
      expect(Character.isDigit('\u064B')).toBeFalsy
      expect(Character.isDigit('\u078A')).toBeFalsy
      expect(Character.isDigit('\u0983')).toBeFalsy
      expect(Character.isDigit('\u0AD0')).toBeFalsy
      expect(Character.isDigit('\u0B3D')).toBeFalsy
      expect(Character.isDigit('\u0E18')).toBeFalsy
      expect(Character.isDigit('\u0F53')).toBeFalsy
      expect(Character.isDigit('\u1433')).toBeFalsy
      expect(Character.isDigit('\u1AD1')).toBeFalsy
      expect(Character.isDigit('\u1D41')).toBeFalsy
      expect(Character.isDigit('\u1FB6')).toBeFalsy
      expect(Character.isDigit('\u2351')).toBeFalsy
      expect(Character.isDigit('\u2F86')).toBeFalsy
      expect(Character.isDigit('\u3198')).toBeFalsy
      expect(Character.isDigit('\u3533')).toBeFalsy
      expect(Character.isDigit('\u3FE1')).toBeFalsy
      expect(Character.isDigit('\u4717')).toBeFalsy
      expect(Character.isDigit('\u47BE')).toBeFalsy
      expect(Character.isDigit('\u4A75')).toBeFalsy
      expect(Character.isDigit('\u4B4E')).toBeFalsy
      expect(Character.isDigit('\u4C18')).toBeFalsy
      expect(Character.isDigit('\u531E')).toBeFalsy
      expect(Character.isDigit('\u5445')).toBeFalsy
      expect(Character.isDigit('\u54CE')).toBeFalsy
      expect(Character.isDigit('\u556E')).toBeFalsy
      expect(Character.isDigit('\u56A6')).toBeFalsy
      expect(Character.isDigit('\u587C')).toBeFalsy
      expect(Character.isDigit('\u5972')).toBeFalsy
      expect(Character.isDigit('\u5A9A')).toBeFalsy
      expect(Character.isDigit('\u5D39')).toBeFalsy
      expect(Character.isDigit('\u60CD')).toBeFalsy
      expect(Character.isDigit('\u60FC')).toBeFalsy
      expect(Character.isDigit('\u611A')).toBeFalsy
      expect(Character.isDigit('\u61DC')).toBeFalsy
      expect(Character.isDigit('\u61EE')).toBeFalsy
      expect(Character.isDigit('\u6750')).toBeFalsy
      expect(Character.isDigit('\u683B')).toBeFalsy
      expect(Character.isDigit('\u6878')).toBeFalsy
      expect(Character.isDigit('\u68A2')).toBeFalsy
      expect(Character.isDigit('\u6B6B')).toBeFalsy
      expect(Character.isDigit('\u6E83')).toBeFalsy
      expect(Character.isDigit('\u7099')).toBeFalsy
      expect(Character.isDigit('\u7717')).toBeFalsy
      expect(Character.isDigit('\u7AAC')).toBeFalsy
      expect(Character.isDigit('\u7B96')).toBeFalsy
      expect(Character.isDigit('\u80C3')).toBeFalsy
      expect(Character.isDigit('\u828F')).toBeFalsy
      expect(Character.isDigit('\u829D')).toBeFalsy
      expect(Character.isDigit('\u876B')).toBeFalsy
      expect(Character.isDigit('\u8913')).toBeFalsy
      expect(Character.isDigit('\u8D2A')).toBeFalsy
      expect(Character.isDigit('\u8D82')).toBeFalsy
      expect(Character.isDigit('\u8E71')).toBeFalsy
      expect(Character.isDigit('\u8F63')).toBeFalsy
      expect(Character.isDigit('\u94D9')).toBeFalsy
      expect(Character.isDigit('\u9859')).toBeFalsy
      expect(Character.isDigit('\u99F9')).toBeFalsy
      expect(Character.isDigit('\u9A1A')).toBeFalsy
      expect(Character.isDigit('\u9B5C')).toBeFalsy
      expect(Character.isDigit('\u9E00')).toBeFalsy
      expect(Character.isDigit('\u9E3B')).toBeFalsy
      expect(Character.isDigit('\u9E5A')).toBeFalsy
      expect(Character.isDigit('\u9F49')).toBeFalsy
      expect(Character.isDigit('\u9FED')).toBeFalsy
      expect(Character.isDigit('\uA28A')).toBeFalsy
      expect(Character.isDigit('\uA6A5')).toBeFalsy
      expect(Character.isDigit('\uA84F')).toBeFalsy
      expect(Character.isDigit('\uA8A1')).toBeFalsy
      expect(Character.isDigit('\uA94C')).toBeFalsy
      expect(Character.isDigit('\uA964')).toBeFalsy
      expect(Character.isDigit('\uABC1')).toBeFalsy
      expect(Character.isDigit('\uAC5A')).toBeFalsy
      expect(Character.isDigit('\uAE22')).toBeFalsy
      expect(Character.isDigit('\uAE82')).toBeFalsy
      expect(Character.isDigit('\uAF4E')).toBeFalsy
      expect(Character.isDigit('\uB680')).toBeFalsy
      expect(Character.isDigit('\uBC1B')).toBeFalsy
      expect(Character.isDigit('\uBDF1')).toBeFalsy
      expect(Character.isDigit('\uC1F3')).toBeFalsy
      expect(Character.isDigit('\uC71B')).toBeFalsy
      expect(Character.isDigit('\uC91F')).toBeFalsy
      expect(Character.isDigit('\uC97F')).toBeFalsy
      expect(Character.isDigit('\uCA08')).toBeFalsy
      expect(Character.isDigit('\uCA4C')).toBeFalsy
      expect(Character.isDigit('\uCCEC')).toBeFalsy
      expect(Character.isDigit('\uCEA2')).toBeFalsy
      expect(Character.isDigit('\uD084')).toBeFalsy
      expect(Character.isDigit('\uD24E')).toBeFalsy
      expect(Character.isDigit('\uD8F5')).toBeFalsy
      expect(Character.isDigit('\uE20A')).toBeFalsy
      expect(Character.isDigit('\uED86')).toBeFalsy
      expect(Character.isDigit('\uEDF0')).toBeFalsy
      expect(Character.isDigit('\uF067')).toBeFalsy
      expect(Character.isDigit('\uF121')).toBeFalsy
      expect(Character.isDigit('\uF448')).toBeFalsy
      expect(Character.isDigit('\uF560')).toBeFalsy
      expect(Character.isDigit('\uF79C')).toBeFalsy
      expect(Character.isDigit('\uF82D')).toBeFalsy
      expect(Character.isDigit('\uFBFC')).toBeFalsy
    }

    it("should provide `compareTo`") {
      def compare(x: Char, y: Char): Int =
        new Character(x).compareTo(new Character(y))

      expect(compare('0', '5')).toBeLessThan(0)
      expect(compare('o', 'g')).toBeGreaterThan(0)
      expect(compare('A', 'a')).toBeLessThan(0)
      expect(compare('b', 'b')).toEqual(0)
    }

    it("should be a Comparable") {
      def compare(x: Any, y: Any): Int =
        x.asInstanceOf[Comparable[Any]].compareTo(y)

      expect(compare('0', '5')).toBeLessThan(0)
      expect(compare('o', 'g')).toBeGreaterThan(0)
      expect(compare('A', 'a')).toBeLessThan(0)
      expect(compare('b', 'b')).toEqual(0)
    }

    it("should provide isIdentifierIgnorable") {
      for (c <- '\u0000' to '\u0008')
        expect(Character.isIdentifierIgnorable(c)).toBeTruthy

      for (c <- '\u000E' to '\u001B')
        expect(Character.isIdentifierIgnorable(c)).toBeTruthy

      for (c <- '\u007F' to '\u009F')
        expect(Character.isIdentifierIgnorable(c)).toBeTruthy

      // Exhaustive list of Cf category. Unicode 7.0.0
      expect(Character.isIdentifierIgnorable('\u00AD')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u0600')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u0601')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u0602')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u0603')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u0604')).toBeTruthy
      // expect(Character.isIdentifierIgnorable('\u0605')).toBeTruthy returns false on JVM
      // expect(Character.isIdentifierIgnorable('\u061C')).toBeTruthy returns false on JVM
      expect(Character.isIdentifierIgnorable('\u06DD')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u070F')).toBeTruthy
      // expect(Character.isIdentifierIgnorable('\u180E')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u200B')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u200C')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u200D')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u200E')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u200F')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u202A')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u202B')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u202C')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u202D')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u202E')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u2060')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u2061')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u2062')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u2063')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u2064')).toBeTruthy
      // expect(Character.isIdentifierIgnorable('\u2066')).toBeTruthy returns false on JVM
      // expect(Character.isIdentifierIgnorable('\u2067')).toBeTruthy returns false on JVM
      // expect(Character.isIdentifierIgnorable('\u2068')).toBeTruthy returns false on JVM
      // expect(Character.isIdentifierIgnorable('\u2069')).toBeTruthy returns false on JVM
      expect(Character.isIdentifierIgnorable('\u206A')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u206B')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u206C')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u206D')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u206E')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\u206F')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\uFEFF')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\uFFF9')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\uFFFA')).toBeTruthy
      expect(Character.isIdentifierIgnorable('\uFFFB')).toBeTruthy

      // BUG in JDK? 17B4 should be "Mn", Java says "Cf"
      //expect(Character.isIdentifierIgnorable('\u17b4')).toBeTruthy

      // 100 randomly generated negatives
      expect(Character.isIdentifierIgnorable('\u745a')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub445')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub23a')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub029')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ufb5c')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u1b67')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u943b')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ue766')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uad12')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub80b')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u7341')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ubc73')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uabb9')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub34b')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u1063')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u272f')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3801')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u53a6')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u2ec2')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u540c')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uc85f')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud2c8')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u551b')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uc0a1')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud25a')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u2b98')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u398b')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ubc77')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u54cc')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uc9a0')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud10f')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uf7e1')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u0f29')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uafcd')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uf187')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u6287')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uacb6')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uff99')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub59e')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uf630')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ufaec')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ua7d7')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3eab')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u54a5')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u393a')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uc621')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u766c')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud64c')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u8beb')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u44e2')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub6f6')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u58b6')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3bad')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3c28')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ufbfd')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u585f')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u7227')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ucea7')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u2c82')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u686d')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u120d')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uf3db')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u320a')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud96e')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u85eb')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u9648')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u08a4')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u9db7')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u82c7')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ufe12')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u0eaf')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u96dc')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3a2a')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uc72e')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3745')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ubcf9')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u5f66')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u9be1')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud81d')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3ca3')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3e82')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u7ce4')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u33ca')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ue725')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uef49')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ue2cf')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\udcf0')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u5f2e')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u2a63')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ud2d2')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u8023')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ua957')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u10ba')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uf85f')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uc40d')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u2509')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u0d8e')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u9db8')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u824d')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u5670')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u6005')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub8de')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uff5c')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\ub36d')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u0cf2')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u82f6')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u9206')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u95e1')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u990f')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u9fc7')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\udffb')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u0ecb')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u7563')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uf0ff')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u6b2e')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u894c')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u8f06')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\uffa9')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u37b0')).toBeFalsy
      expect(Character.isIdentifierIgnorable('\u3e04')).toBeFalsy

    }

    it("should provide isUnicodeIdentifierStart") {
      // 100 randomly generated positives and 100 randomly generated negatives

      expect(Character.isUnicodeIdentifierStart('\ud6d5')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u3f9c')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u3a40')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u53af')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u1636')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u4884')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ucba4')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u1ee4')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u6dec')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u10d4')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u631f')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u3661')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u55f8')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub4ef')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ud509')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u65b5')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u316b')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub270')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7f0f')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uff84')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u11cc')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u0294')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u51b1')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u9ae2')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u304a')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ud5c7')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u3b4b')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u5e42')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u51fc')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uc148')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uc1ae')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7372')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uc116')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u5d29')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u8753')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u50f8')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u3f9d')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u1f44')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ucd43')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u9126')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u8d2e')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u4f5c')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u66d7')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ua30b')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u140b')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub264')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7b35')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u15e4')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ubb37')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u34e3')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uac3e')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ubd0e')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub641')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u1580')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u30c1')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub0c8')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u8681')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7f14')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u4142')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u56c1')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u0444')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u9964')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub5c0')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u43d8')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u479e')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u0853')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ube08')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u9346')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uf9c1')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u0e8a')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u212c')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u810c')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u8089')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u1331')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ua5f7')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u5e5e')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u613b')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u34a7')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ud15b')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\uc1fc')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u92f1')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u3ae6')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ufceb')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7584')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ufe98')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ubb23')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7961')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u4445')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u4d5f')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u61cb')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u5176')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ub987')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u906a')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u4317')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u93ad')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u825a')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u7ff8')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u533a')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\u5617')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ufcc6')).toBeTruthy
      expect(Character.isUnicodeIdentifierStart('\ue398')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ueab6')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue7bc')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf8ab')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue27f')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uebea')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ueedc')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf091')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2785')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u287b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf042')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u20f9')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u23d6')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udc5b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ued16')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u1b6b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue7ba')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf7fa')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2125')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uea97')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue624')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ufbb8')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2730')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udb89')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue30d')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2e24')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf03e')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uda27')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u28fc')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u9ffe')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ude19')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0b70')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uddfc')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ued53')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue8cb')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udccc')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u00a3')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0bed')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0c68')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf47b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0f96')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue9c3')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf784')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uef4b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udee1')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2f61')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf622')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u19f9')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ud86a')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ued83')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf7e4')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uecce')).toBeFalsy

      // BUG in JDK? A699 should be "Ll", Java says "Cn"
      // expect(Character.isUnicodeIdentifierStart('\ua699')).toBeFalsy

      expect(Character.isUnicodeIdentifierStart('\uaa5f')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udf24')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2e0e')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf322')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue137')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ued19')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u21ab')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue972')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udbf2')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf54c')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u4dd3')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2769')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue363')).toBeFalsy

      // BUG in JDK? 1BBB should be "Lo", Java says "Cn"
      // expect(Character.isUnicodeIdentifierStart('\u1bbb')).toBeFalsy

      expect(Character.isUnicodeIdentifierStart('\ueae7')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2bf3')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue704')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u1c7f')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf52b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue9e3')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u259b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf250')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf42f')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ue244')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u20d9')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ua881')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0ee6')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2203')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0fc7')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u07fc')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udb86')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2a70')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2bb7')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uecf0')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ude48')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0a3b')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u20b8')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf898')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u23e6')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ud8ba')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uda1e')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\udc12')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u2a06')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\u0888')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\ud9ec')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf81f')).toBeFalsy
      expect(Character.isUnicodeIdentifierStart('\uf817')).toBeFalsy
    }

    it("should provide isUnicodeIdentifierPart") {
      // 100 randomly generated positives and 100 randomly generated negatives

      expect(Character.isUnicodeIdentifierPart('\u48d3')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u0905')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8f51')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u9bcb')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ud358')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u1538')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\uffcf')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u83ec')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3a89')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ub63a')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ufe24')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u2d62')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u15ca')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u4fa4')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u47d1')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u831c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u84e6')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u7783')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ua03c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6ecf')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u147f')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u67a9')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8b6c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3410')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u2cc0')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ua332')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u9733')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u5df3')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3fd7')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6611')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u55b4')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8bc8')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6f74')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6c97')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6a86')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6000')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u614f')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u206e')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ua801')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u9edf')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ub42c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u7fcd')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8a60')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u182f')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u5d0a')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\uaf9c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u9d4b')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u5088')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\uc1a6')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ubbe4')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\uad25')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u4653')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8add')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3d1c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u80a8')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u810e')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\uc1d2')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ub984')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u9d13')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u37c2')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u13cd')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u53f9')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u98b7')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u57f3')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ub554')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u0176')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ua318')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u9704')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8d52')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u940a')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u0fa5')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u38d1')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3b33')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u93bb')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u03bd')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u4c88')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ud67d')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ubcbf')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3867')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u4368')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8f2d')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u049a')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u4c01')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u5589')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u5e71')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ua1fd')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3a4a')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\uc111')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ub465')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u95af')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ubf2c')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8488')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u4317')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u6b77')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8995')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u7467')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u16b7')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u3ca0')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u5332')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\u8654')).toBeTruthy
      expect(Character.isUnicodeIdentifierPart('\ua8c8')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue3ca')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uebee')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u270e')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf0ac')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue9ec')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u296a')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u33fd')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue5f4')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ueb01')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf38b')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2e6f')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uea69')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf155')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u0f0e')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ueb80')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ud959')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue25e')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf566')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue4a3')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uec44')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u3297')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u3214')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u1bfd')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u4dd0')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uea99')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u309b')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf592')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf4dd')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udfaf')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udd38')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf820')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uaacd')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uff5b')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ude36')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue33b')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udbce')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue1f6')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf78a')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ueb44')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uebd4')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u1df7')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2f10')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u1cbf')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2362')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uebeb')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2ede')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u221d')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2021')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udf41')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u05f5')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u24ab')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uee15')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf175')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf35c')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udc7b')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ud883')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf341')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ueec6')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2f57')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uff64')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue6a4')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uec34')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u22a5')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf5ac')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u3360')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u28b0')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf678')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue0e4')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u233f')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u0afa')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2013')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ud7af')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ud98e')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ud8a5')).toBeFalsy

      // BUG in JDK? A79E should be "Lu", Java says "Cn"
      // expect(Character.isUnicodeIdentifierPart('\ua79e')).toBeFalsy

      expect(Character.isUnicodeIdentifierPart('\u1806')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue07a')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2748')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uabad')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uec5c')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue832')).toBeFalsy

      // BUG in JDK? 08A9 should be "Lo", Java says "Cn"
      // expect(Character.isUnicodeIdentifierPart('\u08a9')).toBeFalsy

      expect(Character.isUnicodeIdentifierPart('\ue4bd')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u208a')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf840')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf570')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uef1e')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2bd4')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue385')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udc18')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u0af0')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u244a')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf01e')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\uf114')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\ue9c4')).toBeFalsy

      // BUG in JDK? AAF4 should be "Lm", Java says "Cn"
      // expect(Character.isUnicodeIdentifierPart('\uaaf4')).toBeFalsy

      expect(Character.isUnicodeIdentifierPart('\uf7b9')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\udd2f')).toBeFalsy
      expect(Character.isUnicodeIdentifierPart('\u2d2c')).toBeFalsy
    }

    it("should provide isUpperCase") {
      for (c <- 'a' to 'z')
        expect(Character.isUpperCase(c)).toBeFalsy
      for (c <- 'A' to 'Z')
        expect(Character.isUpperCase(c)).toBeTruthy
      for (c <- '0' to '9')
        expect(Character.isUpperCase(c)).toBeFalsy

      // example form issue #1646
      expect(Character.isUpperCase('(')).toBeFalsy
      expect(Character.isUpperCase(')')).toBeFalsy

      // 100 randomly chosen uppeer cases
      expect(Character.isUpperCase('\uA78B')).toBeTruthy
      expect(Character.isUpperCase('\u01D1')).toBeTruthy
      expect(Character.isUpperCase('\uA680')).toBeTruthy
      expect(Character.isUpperCase('\u01E2')).toBeTruthy
      expect(Character.isUpperCase('\u00C1')).toBeTruthy
      expect(Character.isUpperCase('\u038F')).toBeTruthy
      expect(Character.isUpperCase('\u04FC')).toBeTruthy
      expect(Character.isUpperCase('\u0232')).toBeTruthy
      expect(Character.isUpperCase('\u1E86')).toBeTruthy
      expect(Character.isUpperCase('\u1E66')).toBeTruthy
      expect(Character.isUpperCase('\u1E12')).toBeTruthy
      expect(Character.isUpperCase('\u1F09')).toBeTruthy
      expect(Character.isUpperCase('\uA758')).toBeTruthy
      expect(Character.isUpperCase('\u01B1')).toBeTruthy
      expect(Character.isUpperCase('\u0230')).toBeTruthy
      expect(Character.isUpperCase('\u2CA2')).toBeTruthy
      expect(Character.isUpperCase('\u0154')).toBeTruthy
      expect(Character.isUpperCase('\uA77E')).toBeTruthy
      expect(Character.isUpperCase('\u2C69')).toBeTruthy
      expect(Character.isUpperCase('\u1EDE')).toBeTruthy
      expect(Character.isUpperCase('\u2CBC')).toBeTruthy
      expect(Character.isUpperCase('\u2CED')).toBeTruthy
      expect(Character.isUpperCase('\u2CAC')).toBeTruthy
      expect(Character.isUpperCase('\u0549')).toBeTruthy
      expect(Character.isUpperCase('\u00D5')).toBeTruthy
      expect(Character.isUpperCase('\u0494')).toBeTruthy
      expect(Character.isUpperCase('\u2CCC')).toBeTruthy
      expect(Character.isUpperCase('\u0162')).toBeTruthy
      expect(Character.isUpperCase('\uFF22')).toBeTruthy
      expect(Character.isUpperCase('\u24C4')).toBeTruthy
      expect(Character.isUpperCase('\u041A')).toBeTruthy
      expect(Character.isUpperCase('\u2124')).toBeTruthy
      expect(Character.isUpperCase('\u1FBB')).toBeTruthy
      expect(Character.isUpperCase('\u1FD9')).toBeTruthy
      expect(Character.isUpperCase('\uA662')).toBeTruthy
      expect(Character.isUpperCase('\u0504')).toBeTruthy
      expect(Character.isUpperCase('\u2C14')).toBeTruthy
      expect(Character.isUpperCase('\uA760')).toBeTruthy
      expect(Character.isUpperCase('\u1E44')).toBeTruthy
      expect(Character.isUpperCase('\u10A2')).toBeTruthy
      expect(Character.isUpperCase('\u017B')).toBeTruthy
      expect(Character.isUpperCase('\u054D')).toBeTruthy
      expect(Character.isUpperCase('\u053E')).toBeTruthy
      expect(Character.isUpperCase('\u1E22')).toBeTruthy
      expect(Character.isUpperCase('\u013F')).toBeTruthy
      expect(Character.isUpperCase('\u1E2A')).toBeTruthy
      expect(Character.isUpperCase('\uA760')).toBeTruthy
      expect(Character.isUpperCase('\u24C3')).toBeTruthy
      expect(Character.isUpperCase('\u049A')).toBeTruthy
      expect(Character.isUpperCase('\u10BD')).toBeTruthy
      expect(Character.isUpperCase('\u1EEA')).toBeTruthy
      expect(Character.isUpperCase('\u0397')).toBeTruthy
      expect(Character.isUpperCase('\u1F3D')).toBeTruthy
      expect(Character.isUpperCase('\u050A')).toBeTruthy
      expect(Character.isUpperCase('\uFF23')).toBeTruthy
      expect(Character.isUpperCase('\u2C01')).toBeTruthy
      expect(Character.isUpperCase('\u10C4')).toBeTruthy
      expect(Character.isUpperCase('\u1EA2')).toBeTruthy
      expect(Character.isUpperCase('\u2C72')).toBeTruthy
      expect(Character.isUpperCase('\u0554')).toBeTruthy
      expect(Character.isUpperCase('\u01E8')).toBeTruthy
      expect(Character.isUpperCase('\u10A4')).toBeTruthy
      expect(Character.isUpperCase('\u1F0C')).toBeTruthy
      expect(Character.isUpperCase('\u0520')).toBeTruthy
      expect(Character.isUpperCase('\u00C5')).toBeTruthy
      expect(Character.isUpperCase('\u10AC')).toBeTruthy
      expect(Character.isUpperCase('\u2CA4')).toBeTruthy
      expect(Character.isUpperCase('\uA73A')).toBeTruthy
      expect(Character.isUpperCase('\u1EFC')).toBeTruthy
      expect(Character.isUpperCase('\u0049')).toBeTruthy
      expect(Character.isUpperCase('\u10A4')).toBeTruthy
      expect(Character.isUpperCase('\uA652')).toBeTruthy
      expect(Character.isUpperCase('\u1E0E')).toBeTruthy
      expect(Character.isUpperCase('\u04E2')).toBeTruthy
      expect(Character.isUpperCase('\u0174')).toBeTruthy
      expect(Character.isUpperCase('\u00CE')).toBeTruthy
      expect(Character.isUpperCase('\u042F')).toBeTruthy
      expect(Character.isUpperCase('\u053F')).toBeTruthy
      expect(Character.isUpperCase('\u0222')).toBeTruthy
      expect(Character.isUpperCase('\u1F3F')).toBeTruthy
      expect(Character.isUpperCase('\uA77E')).toBeTruthy
      expect(Character.isUpperCase('\u01AC')).toBeTruthy
      expect(Character.isUpperCase('\u2C20')).toBeTruthy
      expect(Character.isUpperCase('\u1ECC')).toBeTruthy
      expect(Character.isUpperCase('\u04DE')).toBeTruthy
      expect(Character.isUpperCase('\u004F')).toBeTruthy
      expect(Character.isUpperCase('\u2CE2')).toBeTruthy
      expect(Character.isUpperCase('\u0124')).toBeTruthy
      expect(Character.isUpperCase('\u1F4A')).toBeTruthy
      expect(Character.isUpperCase('\u0112')).toBeTruthy
      expect(Character.isUpperCase('\u016E')).toBeTruthy
      expect(Character.isUpperCase('\u24B7')).toBeTruthy
      expect(Character.isUpperCase('\u211A')).toBeTruthy
      expect(Character.isUpperCase('\uA72A')).toBeTruthy
      expect(Character.isUpperCase('\u0388')).toBeTruthy
      expect(Character.isUpperCase('\uA726')).toBeTruthy
      expect(Character.isUpperCase('\u0052')).toBeTruthy
      expect(Character.isUpperCase('\u1EB0')).toBeTruthy
      expect(Character.isUpperCase('\u0197')).toBeTruthy
      expect(Character.isUpperCase('\u014A')).toBeTruthy

      // 100 randomly chosen non uppeer cases
      expect(Character.isUpperCase('\u58C5')).toBeFalsy
      expect(Character.isUpperCase('\uF319')).toBeFalsy
      expect(Character.isUpperCase('\u1FBC')).toBeFalsy
      expect(Character.isUpperCase('\u8143')).toBeFalsy
      expect(Character.isUpperCase('\u468A')).toBeFalsy
      expect(Character.isUpperCase('\uEAA8')).toBeFalsy
      expect(Character.isUpperCase('\u3D3A')).toBeFalsy
      expect(Character.isUpperCase('\uB168')).toBeFalsy
      expect(Character.isUpperCase('\u6747')).toBeFalsy
      expect(Character.isUpperCase('\uAA8D')).toBeFalsy
      expect(Character.isUpperCase('\u26B7')).toBeFalsy
      expect(Character.isUpperCase('\uDFE3')).toBeFalsy
      expect(Character.isUpperCase('\u51AD')).toBeFalsy
      expect(Character.isUpperCase('\uB4FC')).toBeFalsy
      expect(Character.isUpperCase('\uE23A')).toBeFalsy
      expect(Character.isUpperCase('\u9465')).toBeFalsy
      expect(Character.isUpperCase('\uAE66')).toBeFalsy
      expect(Character.isUpperCase('\u2EA3')).toBeFalsy
      expect(Character.isUpperCase('\u2D05')).toBeFalsy
      expect(Character.isUpperCase('\u2033')).toBeFalsy
      expect(Character.isUpperCase('\u4BFA')).toBeFalsy
      expect(Character.isUpperCase('\u89B6')).toBeFalsy
      expect(Character.isUpperCase('\u9A3F')).toBeFalsy
      expect(Character.isUpperCase('\u4FCD')).toBeFalsy
      expect(Character.isUpperCase('\u770B')).toBeFalsy
      expect(Character.isUpperCase('\u17B4')).toBeFalsy
      expect(Character.isUpperCase('\u08A6')).toBeFalsy
      expect(Character.isUpperCase('\uA202')).toBeFalsy
      expect(Character.isUpperCase('\uE726')).toBeFalsy
      expect(Character.isUpperCase('\u8605')).toBeFalsy
      expect(Character.isUpperCase('\uE24F')).toBeFalsy
      expect(Character.isUpperCase('\u588C')).toBeFalsy
      expect(Character.isUpperCase('\u3AF6')).toBeFalsy
      expect(Character.isUpperCase('\u313F')).toBeFalsy
      expect(Character.isUpperCase('\u8804')).toBeFalsy
      expect(Character.isUpperCase('\uE02A')).toBeFalsy
      expect(Character.isUpperCase('\u89C5')).toBeFalsy
      expect(Character.isUpperCase('\u3B9E')).toBeFalsy
      expect(Character.isUpperCase('\uF678')).toBeFalsy
      expect(Character.isUpperCase('\uD142')).toBeFalsy
      expect(Character.isUpperCase('\u19E5')).toBeFalsy
      expect(Character.isUpperCase('\u8C7A')).toBeFalsy
      expect(Character.isUpperCase('\uBF0C')).toBeFalsy
      expect(Character.isUpperCase('\u5EC6')).toBeFalsy
      expect(Character.isUpperCase('\u6EA3')).toBeFalsy
      expect(Character.isUpperCase('\uCD19')).toBeFalsy
      expect(Character.isUpperCase('\uA6F3')).toBeFalsy
      expect(Character.isUpperCase('\uA5FD')).toBeFalsy
      expect(Character.isUpperCase('\uEAA7')).toBeFalsy
      expect(Character.isUpperCase('\u89DB')).toBeFalsy
      expect(Character.isUpperCase('\u2657')).toBeFalsy
      expect(Character.isUpperCase('\u667A')).toBeFalsy
      expect(Character.isUpperCase('\u131C')).toBeFalsy
      expect(Character.isUpperCase('\u5906')).toBeFalsy
      expect(Character.isUpperCase('\uAA61')).toBeFalsy
      expect(Character.isUpperCase('\uCDD3')).toBeFalsy
      expect(Character.isUpperCase('\u1952')).toBeFalsy
      expect(Character.isUpperCase('\uD93B')).toBeFalsy
      expect(Character.isUpperCase('\uE5B4')).toBeFalsy
      expect(Character.isUpperCase('\uF5D1')).toBeFalsy
      expect(Character.isUpperCase('\uFC49')).toBeFalsy
      expect(Character.isUpperCase('\u51D6')).toBeFalsy
      expect(Character.isUpperCase('\u1D29')).toBeFalsy
      expect(Character.isUpperCase('\u2A30')).toBeFalsy
      expect(Character.isUpperCase('\u881A')).toBeFalsy
      expect(Character.isUpperCase('\u2EA5')).toBeFalsy
      expect(Character.isUpperCase('\u866C')).toBeFalsy
      expect(Character.isUpperCase('\uCAF5')).toBeFalsy
      expect(Character.isUpperCase('\uF5F1')).toBeFalsy
      expect(Character.isUpperCase('\u97B4')).toBeFalsy
      expect(Character.isUpperCase('\uF5F5')).toBeFalsy
      expect(Character.isUpperCase('\u46FD')).toBeFalsy
      expect(Character.isUpperCase('\u538B')).toBeFalsy
      expect(Character.isUpperCase('\u320B')).toBeFalsy
      expect(Character.isUpperCase('\u6F1F')).toBeFalsy
      expect(Character.isUpperCase('\u85B4')).toBeFalsy
      expect(Character.isUpperCase('\u887E')).toBeFalsy
      expect(Character.isUpperCase('\uEA84')).toBeFalsy
      expect(Character.isUpperCase('\uFFE2')).toBeFalsy
      expect(Character.isUpperCase('\u48B2')).toBeFalsy
      expect(Character.isUpperCase('\u3D8D')).toBeFalsy
      expect(Character.isUpperCase('\u071B')).toBeFalsy
      expect(Character.isUpperCase('\u70D9')).toBeFalsy
      expect(Character.isUpperCase('\u86FD')).toBeFalsy
      expect(Character.isUpperCase('\u9303')).toBeFalsy
      expect(Character.isUpperCase('\u6620')).toBeFalsy
      expect(Character.isUpperCase('\uB503')).toBeFalsy
      expect(Character.isUpperCase('\u9D5A')).toBeFalsy
      expect(Character.isUpperCase('\u7909')).toBeFalsy
      expect(Character.isUpperCase('\u772C')).toBeFalsy
      expect(Character.isUpperCase('\u93D8')).toBeFalsy
      expect(Character.isUpperCase('\uE969')).toBeFalsy
      expect(Character.isUpperCase('\u327F')).toBeFalsy
      expect(Character.isUpperCase('\u8473')).toBeFalsy
      expect(Character.isUpperCase('\u478B')).toBeFalsy
      expect(Character.isUpperCase('\uB916')).toBeFalsy
      expect(Character.isUpperCase('\uBBFA')).toBeFalsy
      expect(Character.isUpperCase('\uDC65')).toBeFalsy
      expect(Character.isUpperCase('\u65E9')).toBeFalsy
      expect(Character.isUpperCase('\u97FF')).toBeFalsy
    }

    it("should provide isLowerCase") {
      for (c <- 'a' to 'z')
        expect(Character.isLowerCase(c)).toBeTruthy
      for (c <- 'A' to 'Z')
        expect(Character.isLowerCase(c)).toBeFalsy
      for (c <- '0' to '9')
        expect(Character.isLowerCase(c)).toBeFalsy

      // example form issue #1646
      expect(Character.isLowerCase('(')).toBeFalsy
      expect(Character.isLowerCase(')')).toBeFalsy

      // 50 randomly chosen characters that produce true
      expect(Character.isLowerCase('\u00BA')).toBeTruthy
      expect(Character.isLowerCase('\u02B0')).toBeTruthy
      expect(Character.isLowerCase('\u02B2')).toBeTruthy
      expect(Character.isLowerCase('\u02E1')).toBeTruthy
      expect(Character.isLowerCase('\u02E3')).toBeTruthy
      expect(Character.isLowerCase('\u1D2D')).toBeTruthy
      expect(Character.isLowerCase('\u1D34')).toBeTruthy
      expect(Character.isLowerCase('\u1D36')).toBeTruthy
      expect(Character.isLowerCase('\u1D3B')).toBeTruthy
      expect(Character.isLowerCase('\u1D3D')).toBeTruthy
      expect(Character.isLowerCase('\u1D42')).toBeTruthy
      expect(Character.isLowerCase('\u1D43')).toBeTruthy
      expect(Character.isLowerCase('\u1D48')).toBeTruthy
      expect(Character.isLowerCase('\u1D4C')).toBeTruthy
      expect(Character.isLowerCase('\u1D51')).toBeTruthy
      expect(Character.isLowerCase('\u1D54')).toBeTruthy
      expect(Character.isLowerCase('\u1D61')).toBeTruthy
      expect(Character.isLowerCase('\u1D64')).toBeTruthy
      expect(Character.isLowerCase('\u1D65')).toBeTruthy
      expect(Character.isLowerCase('\u1D66')).toBeTruthy
      expect(Character.isLowerCase('\u1D67')).toBeTruthy
      expect(Character.isLowerCase('\u1D9B')).toBeTruthy
      expect(Character.isLowerCase('\u1D9C')).toBeTruthy
      expect(Character.isLowerCase('\u1DA2')).toBeTruthy
      expect(Character.isLowerCase('\u1DAB')).toBeTruthy
      expect(Character.isLowerCase('\u1DB0')).toBeTruthy
      expect(Character.isLowerCase('\u1DB2')).toBeTruthy
      expect(Character.isLowerCase('\u1DBA')).toBeTruthy
      expect(Character.isLowerCase('\u1DBB')).toBeTruthy
      expect(Character.isLowerCase('\u2092')).toBeTruthy
      expect(Character.isLowerCase('\u2096')).toBeTruthy
      expect(Character.isLowerCase('\u2098')).toBeTruthy
      expect(Character.isLowerCase('\u2099')).toBeTruthy
      expect(Character.isLowerCase('\u2173')).toBeTruthy
      expect(Character.isLowerCase('\u2176')).toBeTruthy
      expect(Character.isLowerCase('\u2177')).toBeTruthy
      expect(Character.isLowerCase('\u217D')).toBeTruthy
      expect(Character.isLowerCase('\u217F')).toBeTruthy
      expect(Character.isLowerCase('\u24D5')).toBeTruthy
      expect(Character.isLowerCase('\u24D9')).toBeTruthy
      expect(Character.isLowerCase('\u24DE')).toBeTruthy
      expect(Character.isLowerCase('\u24E4')).toBeTruthy

      // 50 randomly chosen characters that produce false
      expect(Character.isLowerCase('\u0EB1')).toBeFalsy
      expect(Character.isLowerCase('\u8081')).toBeFalsy
      expect(Character.isLowerCase('\u9CC7')).toBeFalsy
      expect(Character.isLowerCase('\uF690')).toBeFalsy
      expect(Character.isLowerCase(1000364)).toBeFalsy
      expect(Character.isLowerCase(1014512)).toBeFalsy
      expect(Character.isLowerCase(1015728)).toBeFalsy
      expect(Character.isLowerCase(1059585)).toBeFalsy
      expect(Character.isLowerCase(1060648)).toBeFalsy
      expect(Character.isLowerCase(1072459)).toBeFalsy
      expect(Character.isLowerCase(158456)).toBeFalsy
      expect(Character.isLowerCase(202025)).toBeFalsy
      expect(Character.isLowerCase(210769)).toBeFalsy
      expect(Character.isLowerCase(229801)).toBeFalsy
      expect(Character.isLowerCase(297040)).toBeFalsy
      expect(Character.isLowerCase(336465)).toBeFalsy
      expect(Character.isLowerCase(371554)).toBeFalsy
      expect(Character.isLowerCase(375989)).toBeFalsy
      expect(Character.isLowerCase(400272)).toBeFalsy
      expect(Character.isLowerCase(416137)).toBeFalsy
      expect(Character.isLowerCase(438321)).toBeFalsy
      expect(Character.isLowerCase(440405)).toBeFalsy
      expect(Character.isLowerCase(443020)).toBeFalsy
      expect(Character.isLowerCase(459145)).toBeFalsy
      expect(Character.isLowerCase(514186)).toBeFalsy
      expect(Character.isLowerCase(540856)).toBeFalsy
      expect(Character.isLowerCase(596077)).toBeFalsy
      expect(Character.isLowerCase(599610)).toBeFalsy
      expect(Character.isLowerCase(606387)).toBeFalsy
      expect(Character.isLowerCase(669554)).toBeFalsy
      expect(Character.isLowerCase(707507)).toBeFalsy
      expect(Character.isLowerCase(712083)).toBeFalsy
      expect(Character.isLowerCase(725257)).toBeFalsy
      expect(Character.isLowerCase(742132)).toBeFalsy
      expect(Character.isLowerCase(767248)).toBeFalsy
      expect(Character.isLowerCase(767605)).toBeFalsy
      expect(Character.isLowerCase(787060)).toBeFalsy
      expect(Character.isLowerCase(801428)).toBeFalsy
      expect(Character.isLowerCase(816223)).toBeFalsy
      expect(Character.isLowerCase(858580)).toBeFalsy
      expect(Character.isLowerCase(870913)).toBeFalsy
      expect(Character.isLowerCase(907591)).toBeFalsy
      expect(Character.isLowerCase(911179)).toBeFalsy
      expect(Character.isLowerCase(919865)).toBeFalsy
      expect(Character.isLowerCase(923549)).toBeFalsy
      expect(Character.isLowerCase(932196)).toBeFalsy
      expect(Character.isLowerCase(947222)).toBeFalsy
      expect(Character.isLowerCase(967074)).toBeFalsy
      expect(Character.isLowerCase(968024)).toBeFalsy
      expect(Character.isLowerCase(989574)).toBeFalsy
    }

    it("should provide isLetter") {
      for (c <- 'a' to 'z')
        expect(Character.isLetter(c)).toBeTruthy
      for (c <- 'A' to 'Z')
        expect(Character.isLetter(c)).toBeTruthy

      // 100 randomly chosen characters that produce true
      expect(Character.isLetter('\u0931')).toBeTruthy
      expect(Character.isLetter('\u0E2D')).toBeTruthy
      expect(Character.isLetter('\u1E4F')).toBeTruthy
      expect(Character.isLetter('\u2DA5')).toBeTruthy
      expect(Character.isLetter('\u36AF')).toBeTruthy
      expect(Character.isLetter('\u38BE')).toBeTruthy
      expect(Character.isLetter('\u38DD')).toBeTruthy
      expect(Character.isLetter('\u39CE')).toBeTruthy
      expect(Character.isLetter('\u3D44')).toBeTruthy
      expect(Character.isLetter('\u3D63')).toBeTruthy
      expect(Character.isLetter('\u4821')).toBeTruthy
      expect(Character.isLetter('\u53F3')).toBeTruthy
      expect(Character.isLetter('\u5860')).toBeTruthy
      expect(Character.isLetter('\u588E')).toBeTruthy
      expect(Character.isLetter('\u58FE')).toBeTruthy
      expect(Character.isLetter('\u596C')).toBeTruthy
      expect(Character.isLetter('\u59DE')).toBeTruthy
      expect(Character.isLetter('\u670F')).toBeTruthy
      expect(Character.isLetter('\u6E9A')).toBeTruthy
      expect(Character.isLetter('\u7072')).toBeTruthy
      expect(Character.isLetter('\u7542')).toBeTruthy
      expect(Character.isLetter('\u8145')).toBeTruthy
      expect(Character.isLetter('\u8494')).toBeTruthy
      expect(Character.isLetter('\u8495')).toBeTruthy
      expect(Character.isLetter('\u8A95')).toBeTruthy
      expect(Character.isLetter('\u8F1C')).toBeTruthy
      expect(Character.isLetter('\u8FBB')).toBeTruthy
      expect(Character.isLetter('\u9134')).toBeTruthy
      expect(Character.isLetter('\u9503')).toBeTruthy
      expect(Character.isLetter('\u951E')).toBeTruthy
      expect(Character.isLetter('\u9EBA')).toBeTruthy
      expect(Character.isLetter('\uA13C')).toBeTruthy
      expect(Character.isLetter('\uA5E6')).toBeTruthy
      expect(Character.isLetter('\uA810')).toBeTruthy
      expect(Character.isLetter('\uB3B5')).toBeTruthy
      expect(Character.isLetter('\uB4BA')).toBeTruthy
      expect(Character.isLetter('\uB66A')).toBeTruthy
      expect(Character.isLetter('\uCC72')).toBeTruthy
      expect(Character.isLetter('\uCFC5')).toBeTruthy
      expect(Character.isLetter('\uD0EA')).toBeTruthy
      expect(Character.isLetter('\uD3DB')).toBeTruthy
      expect(Character.isLetter('\uD421')).toBeTruthy
      expect(Character.isLetter('\uD4C7')).toBeTruthy
      expect(Character.isLetter('\uD5C9')).toBeTruthy
      expect(Character.isLetter('\uD645')).toBeTruthy
      expect(Character.isLetter('\uFEEC')).toBeTruthy
      expect(Character.isLetter(126614)).toBeTruthy
      expect(Character.isLetter(131284)).toBeTruthy
      expect(Character.isLetter(132125)).toBeTruthy
      expect(Character.isLetter(133027)).toBeTruthy
      expect(Character.isLetter(134132)).toBeTruthy
      expect(Character.isLetter(135009)).toBeTruthy
      expect(Character.isLetter(136707)).toBeTruthy
      expect(Character.isLetter(137302)).toBeTruthy
      expect(Character.isLetter(139777)).toBeTruthy
      expect(Character.isLetter(139817)).toBeTruthy
      expect(Character.isLetter(140291)).toBeTruthy
      expect(Character.isLetter(140662)).toBeTruthy
      expect(Character.isLetter(141121)).toBeTruthy
      expect(Character.isLetter(142226)).toBeTruthy
      expect(Character.isLetter(142357)).toBeTruthy
      expect(Character.isLetter(143348)).toBeTruthy
      expect(Character.isLetter(143406)).toBeTruthy
      expect(Character.isLetter(144320)).toBeTruthy
      expect(Character.isLetter(150432)).toBeTruthy
      expect(Character.isLetter(150647)).toBeTruthy
      expect(Character.isLetter(150718)).toBeTruthy
      expect(Character.isLetter(154065)).toBeTruthy
      expect(Character.isLetter(154216)).toBeTruthy
      expect(Character.isLetter(154406)).toBeTruthy
      expect(Character.isLetter(154981)).toBeTruthy
      expect(Character.isLetter(156075)).toBeTruthy
      expect(Character.isLetter(156323)).toBeTruthy
      expect(Character.isLetter(156813)).toBeTruthy
      expect(Character.isLetter(157428)).toBeTruthy
      expect(Character.isLetter(159375)).toBeTruthy
      expect(Character.isLetter(159743)).toBeTruthy
      expect(Character.isLetter(160356)).toBeTruthy
      expect(Character.isLetter(160611)).toBeTruthy
      expect(Character.isLetter(160671)).toBeTruthy
      expect(Character.isLetter(161921)).toBeTruthy
      expect(Character.isLetter(163295)).toBeTruthy
      expect(Character.isLetter(163637)).toBeTruthy
      expect(Character.isLetter(163690)).toBeTruthy
      expect(Character.isLetter(164083)).toBeTruthy
      expect(Character.isLetter(166120)).toBeTruthy
      expect(Character.isLetter(166691)).toBeTruthy
      expect(Character.isLetter(167425)).toBeTruthy
      expect(Character.isLetter(168930)).toBeTruthy
      expect(Character.isLetter(169746)).toBeTruthy
      expect(Character.isLetter(171107)).toBeTruthy
      expect(Character.isLetter(171334)).toBeTruthy
      expect(Character.isLetter(172074)).toBeTruthy
      expect(Character.isLetter(173301)).toBeTruthy
      expect(Character.isLetter(175989)).toBeTruthy
      expect(Character.isLetter(176347)).toBeTruthy
      expect(Character.isLetter(176935)).toBeTruthy
      expect(Character.isLetter(66253)).toBeTruthy
      expect(Character.isLetter(74015)).toBeTruthy
      expect(Character.isLetter(93997)).toBeTruthy

      // 100 randomly chosen characters that produce false
      expect(Character.isLetter('\uE033')).toBeFalsy
      expect(Character.isLetter('\uF45A')).toBeFalsy
      expect(Character.isLetter(1015529)).toBeFalsy
      expect(Character.isLetter(1037692)).toBeFalsy
      expect(Character.isLetter(1054190)).toBeFalsy
      expect(Character.isLetter(1055278)).toBeFalsy
      expect(Character.isLetter(1063915)).toBeFalsy
      expect(Character.isLetter(1081392)).toBeFalsy
      expect(Character.isLetter(1082345)).toBeFalsy
      expect(Character.isLetter(1112137)).toBeFalsy
      expect(Character.isLetter(112424)).toBeFalsy
      expect(Character.isLetter(119118)).toBeFalsy
      expect(Character.isLetter(127185)).toBeFalsy
      expect(Character.isLetter(181693)).toBeFalsy
      expect(Character.isLetter(185568)).toBeFalsy
      expect(Character.isLetter(191510)).toBeFalsy
      expect(Character.isLetter(205741)).toBeFalsy
      expect(Character.isLetter(208447)).toBeFalsy
      expect(Character.isLetter(217327)).toBeFalsy
      expect(Character.isLetter(223543)).toBeFalsy
      expect(Character.isLetter(226707)).toBeFalsy
      expect(Character.isLetter(250211)).toBeFalsy
      expect(Character.isLetter(252710)).toBeFalsy
      expect(Character.isLetter(270091)).toBeFalsy
      expect(Character.isLetter(275467)).toBeFalsy
      expect(Character.isLetter(282441)).toBeFalsy
      expect(Character.isLetter(289827)).toBeFalsy
      expect(Character.isLetter(293540)).toBeFalsy
      expect(Character.isLetter(306340)).toBeFalsy
      expect(Character.isLetter(310398)).toBeFalsy
      expect(Character.isLetter(313871)).toBeFalsy
      expect(Character.isLetter(325422)).toBeFalsy
      expect(Character.isLetter(330320)).toBeFalsy
      expect(Character.isLetter(331194)).toBeFalsy
      expect(Character.isLetter(353651)).toBeFalsy
      expect(Character.isLetter(354486)).toBeFalsy
      expect(Character.isLetter(358669)).toBeFalsy
      expect(Character.isLetter(359223)).toBeFalsy
      expect(Character.isLetter(369175)).toBeFalsy
      expect(Character.isLetter(401793)).toBeFalsy
      expect(Character.isLetter(427440)).toBeFalsy
      expect(Character.isLetter(429539)).toBeFalsy
      expect(Character.isLetter(442272)).toBeFalsy
      expect(Character.isLetter(447571)).toBeFalsy
      expect(Character.isLetter(465381)).toBeFalsy
      expect(Character.isLetter(471567)).toBeFalsy
      expect(Character.isLetter(480063)).toBeFalsy
      expect(Character.isLetter(485290)).toBeFalsy
      expect(Character.isLetter(494999)).toBeFalsy
      expect(Character.isLetter(506870)).toBeFalsy
      expect(Character.isLetter(518359)).toBeFalsy
      expect(Character.isLetter(535534)).toBeFalsy
      expect(Character.isLetter(539223)).toBeFalsy
      expect(Character.isLetter(545906)).toBeFalsy
      expect(Character.isLetter(548645)).toBeFalsy
      expect(Character.isLetter(555310)).toBeFalsy
      expect(Character.isLetter(557086)).toBeFalsy
      expect(Character.isLetter(557887)).toBeFalsy
      expect(Character.isLetter(564146)).toBeFalsy
      expect(Character.isLetter(565150)).toBeFalsy
      expect(Character.isLetter(567248)).toBeFalsy
      expect(Character.isLetter(567319)).toBeFalsy
      expect(Character.isLetter(572628)).toBeFalsy
      expect(Character.isLetter(578097)).toBeFalsy
      expect(Character.isLetter(604215)).toBeFalsy
      expect(Character.isLetter(627437)).toBeFalsy
      expect(Character.isLetter(645283)).toBeFalsy
      expect(Character.isLetter(645827)).toBeFalsy
      expect(Character.isLetter(646383)).toBeFalsy
      expect(Character.isLetter(690854)).toBeFalsy
      expect(Character.isLetter(695409)).toBeFalsy
      expect(Character.isLetter(698090)).toBeFalsy
      expect(Character.isLetter(703125)).toBeFalsy
      expect(Character.isLetter(713953)).toBeFalsy
      expect(Character.isLetter(723747)).toBeFalsy
      expect(Character.isLetter(729300)).toBeFalsy
      expect(Character.isLetter(732165)).toBeFalsy
      expect(Character.isLetter(732485)).toBeFalsy
      expect(Character.isLetter(737696)).toBeFalsy
      expect(Character.isLetter(742207)).toBeFalsy
      expect(Character.isLetter(743318)).toBeFalsy
      expect(Character.isLetter(749325)).toBeFalsy
      expect(Character.isLetter(761900)).toBeFalsy
      expect(Character.isLetter(763767)).toBeFalsy
      expect(Character.isLetter(769676)).toBeFalsy
      expect(Character.isLetter(775095)).toBeFalsy
      expect(Character.isLetter(792880)).toBeFalsy
      expect(Character.isLetter(797399)).toBeFalsy
      expect(Character.isLetter(80326)).toBeFalsy
      expect(Character.isLetter(816642)).toBeFalsy
      expect(Character.isLetter(837600)).toBeFalsy
      expect(Character.isLetter(839970)).toBeFalsy
      expect(Character.isLetter(844029)).toBeFalsy
      expect(Character.isLetter(86861)).toBeFalsy
      expect(Character.isLetter(873152)).toBeFalsy
      expect(Character.isLetter(884430)).toBeFalsy
      expect(Character.isLetter(900180)).toBeFalsy
      expect(Character.isLetter(901875)).toBeFalsy
      expect(Character.isLetter(916884)).toBeFalsy
      expect(Character.isLetter(983136)).toBeFalsy
    }

    it("should provide isLetterOrDigit") {
      // 100 randomly chosen characters that produce true
      expect(Character.isLetterOrDigit('\u0209')).toBeTruthy
      expect(Character.isLetterOrDigit('\u0377')).toBeTruthy
      expect(Character.isLetterOrDigit('\u0497')).toBeTruthy
      expect(Character.isLetterOrDigit('\u053E')).toBeTruthy
      expect(Character.isLetterOrDigit('\u0726')).toBeTruthy
      expect(Character.isLetterOrDigit('\u075A')).toBeTruthy
      expect(Character.isLetterOrDigit('\u0A91')).toBeTruthy
      expect(Character.isLetterOrDigit('\u12CB')).toBeTruthy
      expect(Character.isLetterOrDigit('\u182B')).toBeTruthy
      expect(Character.isLetterOrDigit('\u1866')).toBeTruthy
      expect(Character.isLetterOrDigit('\u1F1C')).toBeTruthy
      expect(Character.isLetterOrDigit('\u1F74')).toBeTruthy
      expect(Character.isLetterOrDigit('\u1F7C')).toBeTruthy
      expect(Character.isLetterOrDigit('\u2D53')).toBeTruthy
      expect(Character.isLetterOrDigit('\u30DF')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3772')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3A13')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3C1D')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3C4C')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3DEC')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3F1E')).toBeTruthy
      expect(Character.isLetterOrDigit('\u3F4E')).toBeTruthy
      expect(Character.isLetterOrDigit('\u425A')).toBeTruthy
      expect(Character.isLetterOrDigit('\u46D7')).toBeTruthy
      expect(Character.isLetterOrDigit('\u4822')).toBeTruthy
      expect(Character.isLetterOrDigit('\u4855')).toBeTruthy
      expect(Character.isLetterOrDigit('\u485D')).toBeTruthy
      expect(Character.isLetterOrDigit('\u4970')).toBeTruthy
      expect(Character.isLetterOrDigit('\u4C3A')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5188')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5412')).toBeTruthy
      expect(Character.isLetterOrDigit('\u585A')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5933')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5966')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5B15')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5C8B')).toBeTruthy
      expect(Character.isLetterOrDigit('\u5DA2')).toBeTruthy
      expect(Character.isLetterOrDigit('\u6053')).toBeTruthy
      expect(Character.isLetterOrDigit('\u63BB')).toBeTruthy
      expect(Character.isLetterOrDigit('\u6476')).toBeTruthy
      expect(Character.isLetterOrDigit('\u64B9')).toBeTruthy
      expect(Character.isLetterOrDigit('\u65B6')).toBeTruthy
      expect(Character.isLetterOrDigit('\u67EB')).toBeTruthy
      expect(Character.isLetterOrDigit('\u693B')).toBeTruthy
      expect(Character.isLetterOrDigit('\u698C')).toBeTruthy
      expect(Character.isLetterOrDigit('\u6B86')).toBeTruthy
      expect(Character.isLetterOrDigit('\u6DA1')).toBeTruthy
      expect(Character.isLetterOrDigit('\u6E5E')).toBeTruthy
      expect(Character.isLetterOrDigit('\u6EFA')).toBeTruthy
      expect(Character.isLetterOrDigit('\u73BE')).toBeTruthy
      expect(Character.isLetterOrDigit('\u7403')).toBeTruthy
      expect(Character.isLetterOrDigit('\u7611')).toBeTruthy
      expect(Character.isLetterOrDigit('\u76C9')).toBeTruthy
      expect(Character.isLetterOrDigit('\u76F6')).toBeTruthy
      expect(Character.isLetterOrDigit('\u7943')).toBeTruthy
      expect(Character.isLetterOrDigit('\u7A34')).toBeTruthy
      expect(Character.isLetterOrDigit('\u7CA0')).toBeTruthy
      expect(Character.isLetterOrDigit('\u7ED2')).toBeTruthy
      expect(Character.isLetterOrDigit('\u82F6')).toBeTruthy
      expect(Character.isLetterOrDigit('\u83F2')).toBeTruthy
      expect(Character.isLetterOrDigit('\u8401')).toBeTruthy
      expect(Character.isLetterOrDigit('\u8420')).toBeTruthy
      expect(Character.isLetterOrDigit('\u848C')).toBeTruthy
      expect(Character.isLetterOrDigit('\u8602')).toBeTruthy
      expect(Character.isLetterOrDigit('\u8638')).toBeTruthy
      expect(Character.isLetterOrDigit('\u88E7')).toBeTruthy
      expect(Character.isLetterOrDigit('\u8A0B')).toBeTruthy
      expect(Character.isLetterOrDigit('\u9180')).toBeTruthy
      expect(Character.isLetterOrDigit('\u920B')).toBeTruthy
      expect(Character.isLetterOrDigit('\u94C9')).toBeTruthy
      expect(Character.isLetterOrDigit('\u94CE')).toBeTruthy
      expect(Character.isLetterOrDigit('\u95A4')).toBeTruthy
      expect(Character.isLetterOrDigit('\u979E')).toBeTruthy
      expect(Character.isLetterOrDigit('\u9D66')).toBeTruthy
      expect(Character.isLetterOrDigit('\u9ED0')).toBeTruthy
      expect(Character.isLetterOrDigit('\uA047')).toBeTruthy
      expect(Character.isLetterOrDigit('\uA65A')).toBeTruthy
      expect(Character.isLetterOrDigit('\uAEA3')).toBeTruthy
      expect(Character.isLetterOrDigit('\uAEC5')).toBeTruthy
      expect(Character.isLetterOrDigit('\uB583')).toBeTruthy
      expect(Character.isLetterOrDigit('\uBADA')).toBeTruthy
      expect(Character.isLetterOrDigit('\uC56F')).toBeTruthy
      expect(Character.isLetterOrDigit('\uC857')).toBeTruthy
      expect(Character.isLetterOrDigit('\uC97C')).toBeTruthy
      expect(Character.isLetterOrDigit('\uC9BF')).toBeTruthy
      expect(Character.isLetterOrDigit('\uCC2B')).toBeTruthy
      expect(Character.isLetterOrDigit('\uCC60')).toBeTruthy
      expect(Character.isLetterOrDigit('\uCF31')).toBeTruthy
      expect(Character.isLetterOrDigit('\uCF9E')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD2F6')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD353')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD415')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD4B2')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD4D3')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD64E')).toBeTruthy
      expect(Character.isLetterOrDigit('\uD76B')).toBeTruthy
      expect(Character.isLetterOrDigit('\uFA9F')).toBeTruthy
      expect(Character.isLetterOrDigit('\uFB7C')).toBeTruthy
      expect(Character.isLetterOrDigit('\uFCDD')).toBeTruthy
      expect(Character.isLetterOrDigit('\uFEBD')).toBeTruthy

      // 100 randomly chosen characters that produce false
      expect(Character.isLetterOrDigit('\u02D8')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0312')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0361')).toBeFalsy
      expect(Character.isLetterOrDigit('\u05EB')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0618')).toBeFalsy
      expect(Character.isLetterOrDigit('\u070B')).toBeFalsy
      expect(Character.isLetterOrDigit('\u088C')).toBeFalsy
      expect(Character.isLetterOrDigit('\u08D7')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0947')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0B56')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0B77')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0DDA')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0DEB')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0E7B')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0EBA')).toBeFalsy
      expect(Character.isLetterOrDigit('\u0F0B')).toBeFalsy
      expect(Character.isLetterOrDigit('\u109B')).toBeFalsy
      expect(Character.isLetterOrDigit('\u13F7')).toBeFalsy
      expect(Character.isLetterOrDigit('\u181B')).toBeFalsy
      expect(Character.isLetterOrDigit('\u193F')).toBeFalsy
      expect(Character.isLetterOrDigit('\u1FBF')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2103')).toBeFalsy
      expect(Character.isLetterOrDigit('\u219A')).toBeFalsy
      expect(Character.isLetterOrDigit('\u21C4')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2220')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2250')).toBeFalsy
      expect(Character.isLetterOrDigit('\u228D')).toBeFalsy
      expect(Character.isLetterOrDigit('\u231B')).toBeFalsy
      expect(Character.isLetterOrDigit('\u23BC')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2413')).toBeFalsy
      expect(Character.isLetterOrDigit('\u24BF')).toBeFalsy
      expect(Character.isLetterOrDigit('\u258F')).toBeFalsy
      expect(Character.isLetterOrDigit('\u25C2')).toBeFalsy
      expect(Character.isLetterOrDigit('\u27F2')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2887')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2998')).toBeFalsy
      expect(Character.isLetterOrDigit('\u29A5')).toBeFalsy
      expect(Character.isLetterOrDigit('\u29D2')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2A53')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2A70')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2B0E')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2B5E')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2BAF')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2EAE')).toBeFalsy
      expect(Character.isLetterOrDigit('\u2EB1')).toBeFalsy
      expect(Character.isLetterOrDigit('\u32E8')).toBeFalsy
      expect(Character.isLetterOrDigit('\uA7B7')).toBeFalsy
      expect(Character.isLetterOrDigit('\uA7C3')).toBeFalsy
      expect(Character.isLetterOrDigit('\uA837')).toBeFalsy
      expect(Character.isLetterOrDigit('\uA8B4')).toBeFalsy
      expect(Character.isLetterOrDigit('\uA8B7')).toBeFalsy
      expect(Character.isLetterOrDigit('\uAA30')).toBeFalsy
      expect(Character.isLetterOrDigit('\uAAEB')).toBeFalsy
      expect(Character.isLetterOrDigit('\uD859')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDADF')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDC4C')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDC80')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDD5E')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDD9C')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDDC5')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDE97')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDF78')).toBeFalsy
      expect(Character.isLetterOrDigit('\uDF90')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE044')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE0D9')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE158')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE19E')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE2AF')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE317')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE351')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE3C6')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE47F')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE54C')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE67A')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE69C')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE6D1')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE6E2')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE744')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE780')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE7A1')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE817')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE81A')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE8DC')).toBeFalsy
      expect(Character.isLetterOrDigit('\uE916')).toBeFalsy
      expect(Character.isLetterOrDigit('\uEDD3')).toBeFalsy
      expect(Character.isLetterOrDigit('\uEF7C')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF047')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF0D0')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF160')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF27C')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF319')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF46B')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF594')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF6D1')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF7F2')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF889')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF8F6')).toBeFalsy
      expect(Character.isLetterOrDigit('\uF8FF')).toBeFalsy
      expect(Character.isLetterOrDigit('\uFAE2')).toBeFalsy
      expect(Character.isLetterOrDigit('\uFE4B')).toBeFalsy
    }

    it("should provide isSpaceChar") {
      expect(Character.isSpaceChar('\u0020')).toBeTruthy
      expect(Character.isSpaceChar('\u00A0')).toBeTruthy
      expect(Character.isSpaceChar('\u1680')).toBeTruthy
      expect(Character.isSpaceChar('\u180E')).toBeTruthy
      expect(Character.isSpaceChar('\u2000')).toBeTruthy
      expect(Character.isSpaceChar('\u2001')).toBeTruthy
      expect(Character.isSpaceChar('\u2002')).toBeTruthy
      expect(Character.isSpaceChar('\u2004')).toBeTruthy
      expect(Character.isSpaceChar('\u2005')).toBeTruthy
      expect(Character.isSpaceChar('\u2006')).toBeTruthy
      expect(Character.isSpaceChar('\u2007')).toBeTruthy
      expect(Character.isSpaceChar('\u2008')).toBeTruthy
      expect(Character.isSpaceChar('\u2009')).toBeTruthy
      expect(Character.isSpaceChar('\u200A')).toBeTruthy
      expect(Character.isSpaceChar('\u2028')).toBeTruthy
      expect(Character.isSpaceChar('\u2029')).toBeTruthy
      expect(Character.isSpaceChar('\u202F')).toBeTruthy
      expect(Character.isSpaceChar('\u205F')).toBeTruthy
      expect(Character.isSpaceChar('\u3000')).toBeTruthy

      // 100 randomly chosen characters that produce false
      expect(Character.isSpaceChar('\u0057')).toBeFalsy
      expect(Character.isSpaceChar('\u0290')).toBeFalsy
      expect(Character.isSpaceChar('\u0374')).toBeFalsy
      expect(Character.isSpaceChar('\u04EA')).toBeFalsy
      expect(Character.isSpaceChar('\u0845')).toBeFalsy
      expect(Character.isSpaceChar('\u0882')).toBeFalsy
      expect(Character.isSpaceChar('\u0956')).toBeFalsy
      expect(Character.isSpaceChar('\u0979')).toBeFalsy
      expect(Character.isSpaceChar('\u0A78')).toBeFalsy
      expect(Character.isSpaceChar('\u0D6B')).toBeFalsy
      expect(Character.isSpaceChar('\u122F')).toBeFalsy
      expect(Character.isSpaceChar('\u137B')).toBeFalsy
      expect(Character.isSpaceChar('\u1690')).toBeFalsy
      expect(Character.isSpaceChar('\u1875')).toBeFalsy
      expect(Character.isSpaceChar('\u1B96')).toBeFalsy
      expect(Character.isSpaceChar('\u2253')).toBeFalsy
      expect(Character.isSpaceChar('\u2587')).toBeFalsy
      expect(Character.isSpaceChar('\u2770')).toBeFalsy
      expect(Character.isSpaceChar('\u2975')).toBeFalsy
      expect(Character.isSpaceChar('\u304F')).toBeFalsy
      expect(Character.isSpaceChar('\u3567')).toBeFalsy
      expect(Character.isSpaceChar('\u356A')).toBeFalsy
      expect(Character.isSpaceChar('\u35D0')).toBeFalsy
      expect(Character.isSpaceChar('\u3625')).toBeFalsy
      expect(Character.isSpaceChar('\u389E')).toBeFalsy
      expect(Character.isSpaceChar('\u395F')).toBeFalsy
      expect(Character.isSpaceChar('\u3A62')).toBeFalsy
      expect(Character.isSpaceChar('\u3BBF')).toBeFalsy
      expect(Character.isSpaceChar('\u3BF8')).toBeFalsy
      expect(Character.isSpaceChar('\u445D')).toBeFalsy
      expect(Character.isSpaceChar('\u45C1')).toBeFalsy
      expect(Character.isSpaceChar('\u4B46')).toBeFalsy
      expect(Character.isSpaceChar('\u4FBF')).toBeFalsy
      expect(Character.isSpaceChar('\u54B6')).toBeFalsy
      expect(Character.isSpaceChar('\u5A2C')).toBeFalsy
      expect(Character.isSpaceChar('\u5A90')).toBeFalsy
      expect(Character.isSpaceChar('\u5D28')).toBeFalsy
      expect(Character.isSpaceChar('\u5D62')).toBeFalsy
      expect(Character.isSpaceChar('\u5ED9')).toBeFalsy
      expect(Character.isSpaceChar('\u5FA1')).toBeFalsy
      expect(Character.isSpaceChar('\u6242')).toBeFalsy
      expect(Character.isSpaceChar('\u6437')).toBeFalsy
      expect(Character.isSpaceChar('\u673D')).toBeFalsy
      expect(Character.isSpaceChar('\u6AC1')).toBeFalsy
      expect(Character.isSpaceChar('\u6CA1')).toBeFalsy
      expect(Character.isSpaceChar('\u6D31')).toBeFalsy
      expect(Character.isSpaceChar('\u6F7C')).toBeFalsy
      expect(Character.isSpaceChar('\u70B3')).toBeFalsy
      expect(Character.isSpaceChar('\u74F9')).toBeFalsy
      expect(Character.isSpaceChar('\u759E')).toBeFalsy
      expect(Character.isSpaceChar('\u76A0')).toBeFalsy
      expect(Character.isSpaceChar('\u7792')).toBeFalsy
      expect(Character.isSpaceChar('\u814C')).toBeFalsy
      expect(Character.isSpaceChar('\u8209')).toBeFalsy
      expect(Character.isSpaceChar('\u8451')).toBeFalsy
      expect(Character.isSpaceChar('\u86DE')).toBeFalsy
      expect(Character.isSpaceChar('\u8878')).toBeFalsy
      expect(Character.isSpaceChar('\u8BB5')).toBeFalsy
      expect(Character.isSpaceChar('\u8D35')).toBeFalsy
      expect(Character.isSpaceChar('\u903B')).toBeFalsy
      expect(Character.isSpaceChar('\u907C')).toBeFalsy
      expect(Character.isSpaceChar('\u9151')).toBeFalsy
      expect(Character.isSpaceChar('\u9188')).toBeFalsy
      expect(Character.isSpaceChar('\u9775')).toBeFalsy
      expect(Character.isSpaceChar('\u9896')).toBeFalsy
      expect(Character.isSpaceChar('\u9C86')).toBeFalsy
      expect(Character.isSpaceChar('\u9CB2')).toBeFalsy
      expect(Character.isSpaceChar('\uA026')).toBeFalsy
      expect(Character.isSpaceChar('\uA34B')).toBeFalsy
      expect(Character.isSpaceChar('\uA5B2')).toBeFalsy
      expect(Character.isSpaceChar('\uAD41')).toBeFalsy
      expect(Character.isSpaceChar('\uAD56')).toBeFalsy
      expect(Character.isSpaceChar('\uAE7B')).toBeFalsy
      expect(Character.isSpaceChar('\uAFB1')).toBeFalsy
      expect(Character.isSpaceChar('\uB1DE')).toBeFalsy
      expect(Character.isSpaceChar('\uBC4C')).toBeFalsy
      expect(Character.isSpaceChar('\uBE62')).toBeFalsy
      expect(Character.isSpaceChar('\uC041')).toBeFalsy
      expect(Character.isSpaceChar('\uC24F')).toBeFalsy
      expect(Character.isSpaceChar('\uC867')).toBeFalsy
      expect(Character.isSpaceChar('\uC8C1')).toBeFalsy
      expect(Character.isSpaceChar('\uCA16')).toBeFalsy
      expect(Character.isSpaceChar('\uCDDF')).toBeFalsy
      expect(Character.isSpaceChar('\uCF2C')).toBeFalsy
      expect(Character.isSpaceChar('\uD08D')).toBeFalsy
      expect(Character.isSpaceChar('\uD200')).toBeFalsy
      expect(Character.isSpaceChar('\uD599')).toBeFalsy
      expect(Character.isSpaceChar('\uD7F0')).toBeFalsy
      expect(Character.isSpaceChar('\uDBA1')).toBeFalsy
      expect(Character.isSpaceChar('\uDBF8')).toBeFalsy
      expect(Character.isSpaceChar('\uE066')).toBeFalsy
      expect(Character.isSpaceChar('\uE260')).toBeFalsy
      expect(Character.isSpaceChar('\uE776')).toBeFalsy
      expect(Character.isSpaceChar('\uEA9E')).toBeFalsy
      expect(Character.isSpaceChar('\uED5A')).toBeFalsy
      expect(Character.isSpaceChar('\uF016')).toBeFalsy
      expect(Character.isSpaceChar('\uF2E0')).toBeFalsy
      expect(Character.isSpaceChar('\uFA9E')).toBeFalsy
      expect(Character.isSpaceChar('\uFB90')).toBeFalsy
      expect(Character.isSpaceChar('\uFD88')).toBeFalsy
    }

    it("should provide isTitleCase") {
      // all tilteCases
      expect(Character.isTitleCase('\u01C5')).toBeTruthy
      expect(Character.isTitleCase('\u01C8')).toBeTruthy
      expect(Character.isTitleCase('\u01CB')).toBeTruthy
      expect(Character.isTitleCase('\u01F2')).toBeTruthy
      expect(Character.isTitleCase('\u1F88')).toBeTruthy
      expect(Character.isTitleCase('\u1F89')).toBeTruthy
      expect(Character.isTitleCase('\u1F8A')).toBeTruthy
      expect(Character.isTitleCase('\u1F8B')).toBeTruthy
      expect(Character.isTitleCase('\u1F8C')).toBeTruthy
      expect(Character.isTitleCase('\u1F8D')).toBeTruthy
      expect(Character.isTitleCase('\u1F8E')).toBeTruthy
      expect(Character.isTitleCase('\u1F8F')).toBeTruthy
      expect(Character.isTitleCase('\u1F98')).toBeTruthy
      expect(Character.isTitleCase('\u1F99')).toBeTruthy
      expect(Character.isTitleCase('\u1F9A')).toBeTruthy
      expect(Character.isTitleCase('\u1F9B')).toBeTruthy
      expect(Character.isTitleCase('\u1F9C')).toBeTruthy
      expect(Character.isTitleCase('\u1F9D')).toBeTruthy
      expect(Character.isTitleCase('\u1F9E')).toBeTruthy
      expect(Character.isTitleCase('\u1F9F')).toBeTruthy
      expect(Character.isTitleCase('\u1FA8')).toBeTruthy
      expect(Character.isTitleCase('\u1FA9')).toBeTruthy
      expect(Character.isTitleCase('\u1FAA')).toBeTruthy
      expect(Character.isTitleCase('\u1FAB')).toBeTruthy
      expect(Character.isTitleCase('\u1FAC')).toBeTruthy
      expect(Character.isTitleCase('\u1FAD')).toBeTruthy
      expect(Character.isTitleCase('\u1FAE')).toBeTruthy
      expect(Character.isTitleCase('\u1FAF')).toBeTruthy
      expect(Character.isTitleCase('\u1FBC')).toBeTruthy
      expect(Character.isTitleCase('\u1FCC')).toBeTruthy
      expect(Character.isTitleCase('\u1FFC')).toBeTruthy

      // 100 randomly chosen characters that produce false
      expect(Character.isTitleCase('\u043A')).toBeFalsy
      expect(Character.isTitleCase('\u09F9')).toBeFalsy
      expect(Character.isTitleCase('\u0BD0')).toBeFalsy
      expect(Character.isTitleCase('\u0DF3')).toBeFalsy
      expect(Character.isTitleCase('\u1290')).toBeFalsy
      expect(Character.isTitleCase('\u1A35')).toBeFalsy
      expect(Character.isTitleCase('\u1B86')).toBeFalsy
      expect(Character.isTitleCase('\u1DB3')).toBeFalsy
      expect(Character.isTitleCase('\u228B')).toBeFalsy
      expect(Character.isTitleCase('\u22D1')).toBeFalsy
      expect(Character.isTitleCase('\u231A')).toBeFalsy
      expect(Character.isTitleCase('\u2A69')).toBeFalsy
      expect(Character.isTitleCase('\u2AD3')).toBeFalsy
      expect(Character.isTitleCase('\u2B75')).toBeFalsy
      expect(Character.isTitleCase('\u2FDC')).toBeFalsy
      expect(Character.isTitleCase('\u323F')).toBeFalsy
      expect(Character.isTitleCase('\u3354')).toBeFalsy
      expect(Character.isTitleCase('\u35AA')).toBeFalsy
      expect(Character.isTitleCase('\u3E32')).toBeFalsy
      expect(Character.isTitleCase('\u3EF6')).toBeFalsy
      expect(Character.isTitleCase('\u4251')).toBeFalsy
      expect(Character.isTitleCase('\u4860')).toBeFalsy
      expect(Character.isTitleCase('\u4CF7')).toBeFalsy
      expect(Character.isTitleCase('\u50C7')).toBeFalsy
      expect(Character.isTitleCase('\u531E')).toBeFalsy
      expect(Character.isTitleCase('\u54E9')).toBeFalsy
      expect(Character.isTitleCase('\u57F4')).toBeFalsy
      expect(Character.isTitleCase('\u5870')).toBeFalsy
      expect(Character.isTitleCase('\u58BF')).toBeFalsy
      expect(Character.isTitleCase('\u599B')).toBeFalsy
      expect(Character.isTitleCase('\u5D43')).toBeFalsy
      expect(Character.isTitleCase('\u62DC')).toBeFalsy
      expect(Character.isTitleCase('\u6BCA')).toBeFalsy
      expect(Character.isTitleCase('\u6DB8')).toBeFalsy
      expect(Character.isTitleCase('\u711E')).toBeFalsy
      expect(Character.isTitleCase('\u75E4')).toBeFalsy
      expect(Character.isTitleCase('\u7926')).toBeFalsy
      expect(Character.isTitleCase('\u7BBD')).toBeFalsy
      expect(Character.isTitleCase('\u7C46')).toBeFalsy
      expect(Character.isTitleCase('\u7CA5')).toBeFalsy
      expect(Character.isTitleCase('\u7E88')).toBeFalsy
      expect(Character.isTitleCase('\u7FA9')).toBeFalsy
      expect(Character.isTitleCase('\u817A')).toBeFalsy
      expect(Character.isTitleCase('\u82BD')).toBeFalsy
      expect(Character.isTitleCase('\u82F0')).toBeFalsy
      expect(Character.isTitleCase('\u8435')).toBeFalsy
      expect(Character.isTitleCase('\u85A5')).toBeFalsy
      expect(Character.isTitleCase('\u9080')).toBeFalsy
      expect(Character.isTitleCase('\u93BF')).toBeFalsy
      expect(Character.isTitleCase('\u9A88')).toBeFalsy
      expect(Character.isTitleCase('\u9AF9')).toBeFalsy
      expect(Character.isTitleCase('\uA18C')).toBeFalsy
      expect(Character.isTitleCase('\uA5C4')).toBeFalsy
      expect(Character.isTitleCase('\uA91B')).toBeFalsy
      expect(Character.isTitleCase('\uAD45')).toBeFalsy
      expect(Character.isTitleCase('\uAE88')).toBeFalsy
      expect(Character.isTitleCase('\uB06F')).toBeFalsy
      expect(Character.isTitleCase('\uB2E0')).toBeFalsy
      expect(Character.isTitleCase('\uB427')).toBeFalsy
      expect(Character.isTitleCase('\uB512')).toBeFalsy
      expect(Character.isTitleCase('\uB59C')).toBeFalsy
      expect(Character.isTitleCase('\uB79B')).toBeFalsy
      expect(Character.isTitleCase('\uB911')).toBeFalsy
      expect(Character.isTitleCase('\uB995')).toBeFalsy
      expect(Character.isTitleCase('\uBA25')).toBeFalsy
      expect(Character.isTitleCase('\uBFED')).toBeFalsy
      expect(Character.isTitleCase('\uC0E1')).toBeFalsy
      expect(Character.isTitleCase('\uC5CC')).toBeFalsy
      expect(Character.isTitleCase('\uC80B')).toBeFalsy
      expect(Character.isTitleCase('\uC823')).toBeFalsy
      expect(Character.isTitleCase('\uCAE2')).toBeFalsy
      expect(Character.isTitleCase('\uCC2E')).toBeFalsy
      expect(Character.isTitleCase('\uCD96')).toBeFalsy
      expect(Character.isTitleCase('\uD025')).toBeFalsy
      expect(Character.isTitleCase('\uD270')).toBeFalsy
      expect(Character.isTitleCase('\uD3B2')).toBeFalsy
      expect(Character.isTitleCase('\uD679')).toBeFalsy
      expect(Character.isTitleCase('\uD822')).toBeFalsy
      expect(Character.isTitleCase('\uDA58')).toBeFalsy
      expect(Character.isTitleCase('\uE1FE')).toBeFalsy
      expect(Character.isTitleCase('\uE247')).toBeFalsy
      expect(Character.isTitleCase('\uE41E')).toBeFalsy
      expect(Character.isTitleCase('\uE518')).toBeFalsy
      expect(Character.isTitleCase('\uE6F1')).toBeFalsy
      expect(Character.isTitleCase('\uE7E8')).toBeFalsy
      expect(Character.isTitleCase('\uE89F')).toBeFalsy
      expect(Character.isTitleCase('\uEA41')).toBeFalsy
      expect(Character.isTitleCase('\uEA6D')).toBeFalsy
      expect(Character.isTitleCase('\uEAB6')).toBeFalsy
      expect(Character.isTitleCase('\uEBD4')).toBeFalsy
      expect(Character.isTitleCase('\uEF15')).toBeFalsy
      expect(Character.isTitleCase('\uF506')).toBeFalsy
      expect(Character.isTitleCase('\uF519')).toBeFalsy
      expect(Character.isTitleCase('\uF53F')).toBeFalsy
      expect(Character.isTitleCase('\uF5EC')).toBeFalsy
      expect(Character.isTitleCase('\uF62F')).toBeFalsy
      expect(Character.isTitleCase('\uFAD4')).toBeFalsy
      expect(Character.isTitleCase('\uFDBC')).toBeFalsy
      expect(Character.isTitleCase('\u0029')).toBeFalsy
      expect(Character.isTitleCase('\u0070')).toBeFalsy
    }

    it("should provide isMirrored") {
      // 50 randomly chosen characters that produce true
      expect(Character.isMirrored('\u208D')).toBeTruthy
      expect(Character.isMirrored('\u208E')).toBeTruthy
      expect(Character.isMirrored('\u2140')).toBeTruthy
      expect(Character.isMirrored('\u2201')).toBeTruthy
      expect(Character.isMirrored('\u2204')).toBeTruthy
      expect(Character.isMirrored('\u221A')).toBeTruthy
      expect(Character.isMirrored('\u221B')).toBeTruthy
      expect(Character.isMirrored('\u221D')).toBeTruthy
      expect(Character.isMirrored('\u222D')).toBeTruthy
      expect(Character.isMirrored('\u2232')).toBeTruthy
      expect(Character.isMirrored('\u2252')).toBeTruthy
      expect(Character.isMirrored('\u2254')).toBeTruthy
      expect(Character.isMirrored('\u226A')).toBeTruthy
      expect(Character.isMirrored('\u2271')).toBeTruthy
      expect(Character.isMirrored('\u227A')).toBeTruthy
      expect(Character.isMirrored('\u2281')).toBeTruthy
      expect(Character.isMirrored('\u2284')).toBeTruthy
      expect(Character.isMirrored('\u2286')).toBeTruthy
      expect(Character.isMirrored('\u22A2')).toBeTruthy
      expect(Character.isMirrored('\u22D0')).toBeTruthy
      expect(Character.isMirrored('\u22DA')).toBeTruthy
      expect(Character.isMirrored('\u22F1')).toBeTruthy
      expect(Character.isMirrored('\u22F2')).toBeTruthy
      expect(Character.isMirrored('\u22F9')).toBeTruthy
      expect(Character.isMirrored('\u27C3')).toBeTruthy
      expect(Character.isMirrored('\u27CD')).toBeTruthy
      expect(Character.isMirrored('\u27EA')).toBeTruthy
      expect(Character.isMirrored('\u2988')).toBeTruthy
      expect(Character.isMirrored('\u298D')).toBeTruthy
      expect(Character.isMirrored('\u299C')).toBeTruthy
      expect(Character.isMirrored('\u29A5')).toBeTruthy
      expect(Character.isMirrored('\u29D1')).toBeTruthy
      expect(Character.isMirrored('\u29D9')).toBeTruthy
      expect(Character.isMirrored('\u29E4')).toBeTruthy
      expect(Character.isMirrored('\u29F5')).toBeTruthy
      expect(Character.isMirrored('\u2A1C')).toBeTruthy
      expect(Character.isMirrored('\u2A89')).toBeTruthy
      expect(Character.isMirrored('\u2A94')).toBeTruthy
      expect(Character.isMirrored('\u2AAB')).toBeTruthy
      expect(Character.isMirrored('\u2ABE')).toBeTruthy
      expect(Character.isMirrored('\u2AF9')).toBeTruthy
      expect(Character.isMirrored('\u2AFD')).toBeTruthy
      expect(Character.isMirrored('\u2E02')).toBeTruthy
      expect(Character.isMirrored('\u2E23')).toBeTruthy
      expect(Character.isMirrored('\u2E24')).toBeTruthy
      expect(Character.isMirrored('\uFE5C')).toBeTruthy
      expect(Character.isMirrored('\uFE64')).toBeTruthy

      // 50 randomly chosen characters that produce false
      expect(Character.isMirrored('\u0688')).toBeFalsy
      expect(Character.isMirrored('\u9319')).toBeFalsy
      expect(Character.isMirrored('\uFAF5')).toBeFalsy
      expect(Character.isMirrored(1030807)).toBeFalsy
      expect(Character.isMirrored(1032997)).toBeFalsy
      expect(Character.isMirrored(1073298)).toBeFalsy
      expect(Character.isMirrored(1087805)).toBeFalsy
      expect(Character.isMirrored(1090056)).toBeFalsy
      expect(Character.isMirrored(1106453)).toBeFalsy
      expect(Character.isMirrored(112391)).toBeFalsy
      expect(Character.isMirrored(114041)).toBeFalsy
      expect(Character.isMirrored(135408)).toBeFalsy
      expect(Character.isMirrored(139769)).toBeFalsy
      expect(Character.isMirrored(141057)).toBeFalsy
      expect(Character.isMirrored(141146)).toBeFalsy
      expect(Character.isMirrored(214605)).toBeFalsy
      expect(Character.isMirrored(245103)).toBeFalsy
      expect(Character.isMirrored(245550)).toBeFalsy
      expect(Character.isMirrored(265448)).toBeFalsy
      expect(Character.isMirrored(289372)).toBeFalsy
      expect(Character.isMirrored(290850)).toBeFalsy
      expect(Character.isMirrored(314422)).toBeFalsy
      expect(Character.isMirrored(360533)).toBeFalsy
      expect(Character.isMirrored(361613)).toBeFalsy
      expect(Character.isMirrored(399166)).toBeFalsy
      expect(Character.isMirrored(418562)).toBeFalsy
      expect(Character.isMirrored(441100)).toBeFalsy
      expect(Character.isMirrored(466467)).toBeFalsy
      expect(Character.isMirrored(478964)).toBeFalsy
      expect(Character.isMirrored(528293)).toBeFalsy
      expect(Character.isMirrored(533681)).toBeFalsy
      expect(Character.isMirrored(557036)).toBeFalsy
      expect(Character.isMirrored(599773)).toBeFalsy
      expect(Character.isMirrored(666210)).toBeFalsy
      expect(Character.isMirrored(67175)).toBeFalsy
      expect(Character.isMirrored(673481)).toBeFalsy
      expect(Character.isMirrored(714304)).toBeFalsy
      expect(Character.isMirrored(725335)).toBeFalsy
      expect(Character.isMirrored(746217)).toBeFalsy
      expect(Character.isMirrored(74649)).toBeFalsy
      expect(Character.isMirrored(777837)).toBeFalsy
      expect(Character.isMirrored(808419)).toBeFalsy
      expect(Character.isMirrored(829990)).toBeFalsy
      expect(Character.isMirrored(83437)).toBeFalsy
      expect(Character.isMirrored(907564)).toBeFalsy
      expect(Character.isMirrored(918918)).toBeFalsy
      expect(Character.isMirrored(921306)).toBeFalsy
      expect(Character.isMirrored(925902)).toBeFalsy
      expect(Character.isMirrored(927621)).toBeFalsy
      expect(Character.isMirrored(962242)).toBeFalsy
    }

    it("should provide isDefined") {
      // 100 randomly chosen characters that produce true
      expect(Character.isDefined('\u0191')).toBeTruthy
      expect(Character.isDefined('\u03D6')).toBeTruthy
      expect(Character.isDefined('\u04A1')).toBeTruthy
      expect(Character.isDefined('\u04BD')).toBeTruthy
      expect(Character.isDefined('\u07AC')).toBeTruthy
      expect(Character.isDefined('\u0BED')).toBeTruthy
      expect(Character.isDefined('\u0D21')).toBeTruthy
      expect(Character.isDefined('\u0D6B')).toBeTruthy
      expect(Character.isDefined('\u106D')).toBeTruthy
      expect(Character.isDefined('\u1982')).toBeTruthy
      expect(Character.isDefined('\u1A46')).toBeTruthy
      expect(Character.isDefined('\u1CD5')).toBeTruthy
      expect(Character.isDefined('\u1E87')).toBeTruthy
      expect(Character.isDefined('\u206C')).toBeTruthy
      expect(Character.isDefined('\u22F6')).toBeTruthy
      expect(Character.isDefined('\u2325')).toBeTruthy
      expect(Character.isDefined('\u277C')).toBeTruthy
      expect(Character.isDefined('\u2E0D')).toBeTruthy
      expect(Character.isDefined('\u2E27')).toBeTruthy
      expect(Character.isDefined('\u2FA4')).toBeTruthy
      expect(Character.isDefined('\u3163')).toBeTruthy
      expect(Character.isDefined('\u32D7')).toBeTruthy
      expect(Character.isDefined('\u3384')).toBeTruthy
      expect(Character.isDefined('\u33D3')).toBeTruthy
      expect(Character.isDefined('\u35D9')).toBeTruthy
      expect(Character.isDefined('\u36F2')).toBeTruthy
      expect(Character.isDefined('\u385B')).toBeTruthy
      expect(Character.isDefined('\u398D')).toBeTruthy
      expect(Character.isDefined('\u3A81')).toBeTruthy
      expect(Character.isDefined('\u3D57')).toBeTruthy
      expect(Character.isDefined('\u3DD4')).toBeTruthy
      expect(Character.isDefined('\u415C')).toBeTruthy
      expect(Character.isDefined('\u4233')).toBeTruthy
      expect(Character.isDefined('\u4A87')).toBeTruthy
      expect(Character.isDefined('\u50C7')).toBeTruthy
      expect(Character.isDefined('\u51A2')).toBeTruthy
      expect(Character.isDefined('\u525B')).toBeTruthy
      expect(Character.isDefined('\u62D0')).toBeTruthy
      expect(Character.isDefined('\u6405')).toBeTruthy
      expect(Character.isDefined('\u67D2')).toBeTruthy
      expect(Character.isDefined('\u6ED4')).toBeTruthy
      expect(Character.isDefined('\u6F13')).toBeTruthy
      expect(Character.isDefined('\u7366')).toBeTruthy
      expect(Character.isDefined('\u7661')).toBeTruthy
      expect(Character.isDefined('\u76FB')).toBeTruthy
      expect(Character.isDefined('\u77D7')).toBeTruthy
      expect(Character.isDefined('\u7A26')).toBeTruthy
      expect(Character.isDefined('\u7E41')).toBeTruthy
      expect(Character.isDefined('\u7E9E')).toBeTruthy
      expect(Character.isDefined('\u7F5E')).toBeTruthy
      expect(Character.isDefined('\u8336')).toBeTruthy
      expect(Character.isDefined('\u842D')).toBeTruthy
      expect(Character.isDefined('\u8488')).toBeTruthy
      expect(Character.isDefined('\u88C9')).toBeTruthy
      expect(Character.isDefined('\u8A81')).toBeTruthy
      expect(Character.isDefined('\u8D21')).toBeTruthy
      expect(Character.isDefined('\u8EA0')).toBeTruthy
      expect(Character.isDefined('\u921B')).toBeTruthy
      expect(Character.isDefined('\u94EC')).toBeTruthy
      expect(Character.isDefined('\u972C')).toBeTruthy
      expect(Character.isDefined('\u98D4')).toBeTruthy
      expect(Character.isDefined('\u993D')).toBeTruthy
      expect(Character.isDefined('\u9B13')).toBeTruthy
      expect(Character.isDefined('\u9B6F')).toBeTruthy
      expect(Character.isDefined('\uA278')).toBeTruthy
      expect(Character.isDefined('\uA2F9')).toBeTruthy
      expect(Character.isDefined('\uA618')).toBeTruthy
      expect(Character.isDefined('\uA789')).toBeTruthy
      expect(Character.isDefined('\uAA1C')).toBeTruthy
      expect(Character.isDefined('\uAEF8')).toBeTruthy
      expect(Character.isDefined('\uAF74')).toBeTruthy
      expect(Character.isDefined('\uB170')).toBeTruthy
      expect(Character.isDefined('\uB508')).toBeTruthy
      expect(Character.isDefined('\uB820')).toBeTruthy
      expect(Character.isDefined('\uB8E3')).toBeTruthy
      expect(Character.isDefined('\uBC34')).toBeTruthy
      expect(Character.isDefined('\uC501')).toBeTruthy
      expect(Character.isDefined('\uCABF')).toBeTruthy
      expect(Character.isDefined('\uCC21')).toBeTruthy
      expect(Character.isDefined('\uCC27')).toBeTruthy
      expect(Character.isDefined('\uCC46')).toBeTruthy
      expect(Character.isDefined('\uCC7F')).toBeTruthy
      expect(Character.isDefined('\uCD32')).toBeTruthy
      expect(Character.isDefined('\uCFE0')).toBeTruthy
      expect(Character.isDefined('\uD08F')).toBeTruthy
      expect(Character.isDefined('\uD4AF')).toBeTruthy
      expect(Character.isDefined('\uD522')).toBeTruthy
      expect(Character.isDefined('\uD9F4')).toBeTruthy
      expect(Character.isDefined('\uE5A7')).toBeTruthy
      expect(Character.isDefined('\uEB96')).toBeTruthy
      expect(Character.isDefined('\uEEC0')).toBeTruthy
      expect(Character.isDefined('\uEF30')).toBeTruthy
      expect(Character.isDefined('\uEFD9')).toBeTruthy
      expect(Character.isDefined('\uF4E8')).toBeTruthy
      expect(Character.isDefined('\uF547')).toBeTruthy
      expect(Character.isDefined('\uFA7D')).toBeTruthy
      expect(Character.isDefined('\uFD39')).toBeTruthy
      expect(Character.isDefined('\uFF3A')).toBeTruthy

      // 100 randomly chosen characters that produce false
      expect(Character.isDefined('\u0528')).toBeFalsy
      expect(Character.isDefined('\u052B')).toBeFalsy
      expect(Character.isDefined('\u058D')).toBeFalsy
      expect(Character.isDefined('\u05FE')).toBeFalsy
      expect(Character.isDefined('\u07BE')).toBeFalsy
      expect(Character.isDefined('\u07FB')).toBeFalsy
      expect(Character.isDefined('\u0878')).toBeFalsy
      expect(Character.isDefined('\u0879')).toBeFalsy
      expect(Character.isDefined('\u088E')).toBeFalsy
      expect(Character.isDefined('\u0892')).toBeFalsy
      expect(Character.isDefined('\u08BE')).toBeFalsy
      expect(Character.isDefined('\u08C5')).toBeFalsy
      expect(Character.isDefined('\u08D4')).toBeFalsy
      expect(Character.isDefined('\u08DB')).toBeFalsy
      expect(Character.isDefined('\u08E1')).toBeFalsy
      expect(Character.isDefined('\u09B1')).toBeFalsy
      expect(Character.isDefined('\u09D0')).toBeFalsy
      expect(Character.isDefined('\u09D5')).toBeFalsy
      expect(Character.isDefined('\u09FF')).toBeFalsy
      expect(Character.isDefined('\u0A12')).toBeFalsy
      expect(Character.isDefined('\u0A52')).toBeFalsy
      expect(Character.isDefined('\u0A60')).toBeFalsy
      expect(Character.isDefined('\u0AE4')).toBeFalsy
      expect(Character.isDefined('\u0AF4')).toBeFalsy
      expect(Character.isDefined('\u0B3B')).toBeFalsy
      expect(Character.isDefined('\u0B7F')).toBeFalsy
      expect(Character.isDefined('\u0BBA')).toBeFalsy
      expect(Character.isDefined('\u0BBD')).toBeFalsy
      expect(Character.isDefined('\u0CDF')).toBeFalsy
      expect(Character.isDefined('\u0D49')).toBeFalsy
      expect(Character.isDefined('\u0D99')).toBeFalsy
      expect(Character.isDefined('\u0DBC')).toBeFalsy
      expect(Character.isDefined('\u0E00')).toBeFalsy
      expect(Character.isDefined('\u0E74')).toBeFalsy
      expect(Character.isDefined('\u0E8E')).toBeFalsy
      expect(Character.isDefined('\u0E93')).toBeFalsy
      expect(Character.isDefined('\u0EA8')).toBeFalsy
      expect(Character.isDefined('\u0EF6')).toBeFalsy
      expect(Character.isDefined('\u0FDB')).toBeFalsy
      expect(Character.isDefined('\u0FE5')).toBeFalsy
      expect(Character.isDefined('\u10C9')).toBeFalsy
      expect(Character.isDefined('\u12C6')).toBeFalsy
      expect(Character.isDefined('\u16F7')).toBeFalsy
      expect(Character.isDefined('\u16FA')).toBeFalsy
      expect(Character.isDefined('\u1739')).toBeFalsy
      expect(Character.isDefined('\u173A')).toBeFalsy
      expect(Character.isDefined('\u177A')).toBeFalsy
      expect(Character.isDefined('\u17DF')).toBeFalsy
      expect(Character.isDefined('\u187A')).toBeFalsy
      expect(Character.isDefined('\u1C39')).toBeFalsy
      expect(Character.isDefined('\u1CAC')).toBeFalsy
      expect(Character.isDefined('\u1CAF')).toBeFalsy
      expect(Character.isDefined('\u1CBB')).toBeFalsy
      expect(Character.isDefined('\u1CC9')).toBeFalsy
      expect(Character.isDefined('\u1FF1')).toBeFalsy
      expect(Character.isDefined('\u2073')).toBeFalsy
      expect(Character.isDefined('\u23F7')).toBeFalsy
      expect(Character.isDefined('\u23F8')).toBeFalsy
      expect(Character.isDefined('\u243F')).toBeFalsy
      expect(Character.isDefined('\u2700')).toBeFalsy
      expect(Character.isDefined('\u2B63')).toBeFalsy
      expect(Character.isDefined('\u2B7F')).toBeFalsy
      expect(Character.isDefined('\u2BDC')).toBeFalsy
      expect(Character.isDefined('\u2BF5')).toBeFalsy
      expect(Character.isDefined('\u2BFE')).toBeFalsy
      expect(Character.isDefined('\u2C2F')).toBeFalsy
      expect(Character.isDefined('\u2EFE')).toBeFalsy
      expect(Character.isDefined('\u2FE9')).toBeFalsy
      expect(Character.isDefined('\u2FEE')).toBeFalsy
      expect(Character.isDefined('\u31EE')).toBeFalsy
      expect(Character.isDefined('\u32FF')).toBeFalsy
      expect(Character.isDefined('\u9FE9')).toBeFalsy
      expect(Character.isDefined('\uA639')).toBeFalsy
      expect(Character.isDefined('\uA7AE')).toBeFalsy
      expect(Character.isDefined('\uA7EC')).toBeFalsy
      expect(Character.isDefined('\uA7F2')).toBeFalsy
      expect(Character.isDefined('\uA8C7')).toBeFalsy
      expect(Character.isDefined('\uA8DD')).toBeFalsy
      expect(Character.isDefined('\uA8DF')).toBeFalsy
      expect(Character.isDefined('\uA957')).toBeFalsy
      expect(Character.isDefined('\uAAC6')).toBeFalsy
      expect(Character.isDefined('\uAAC9')).toBeFalsy
      expect(Character.isDefined('\uAAD9')).toBeFalsy
      expect(Character.isDefined('\uAAF9')).toBeFalsy
      expect(Character.isDefined('\uAB46')).toBeFalsy
      expect(Character.isDefined('\uAB5B')).toBeFalsy
      expect(Character.isDefined('\uAB70')).toBeFalsy
      expect(Character.isDefined('\uAB95')).toBeFalsy
      expect(Character.isDefined('\uABA7')).toBeFalsy
      expect(Character.isDefined('\uD7C9')).toBeFalsy
      expect(Character.isDefined('\uFAE1')).toBeFalsy
      expect(Character.isDefined('\uFAE9')).toBeFalsy
      expect(Character.isDefined('\uFB12')).toBeFalsy
      expect(Character.isDefined('\uFBCA')).toBeFalsy
      expect(Character.isDefined('\uFD40')).toBeFalsy
      expect(Character.isDefined('\uFFF3')).toBeFalsy
    }

    it("should provide getType") {
      // 300 first characters
      expect(Character.getType(0)).toEqual(15)
      expect(Character.getType(1)).toEqual(15)
      expect(Character.getType(2)).toEqual(15)
      expect(Character.getType(3)).toEqual(15)
      expect(Character.getType(4)).toEqual(15)
      expect(Character.getType(5)).toEqual(15)
      expect(Character.getType(6)).toEqual(15)
      expect(Character.getType(7)).toEqual(15)
      expect(Character.getType(8)).toEqual(15)
      expect(Character.getType(9)).toEqual(15)
      expect(Character.getType(10)).toEqual(15)
      expect(Character.getType(11)).toEqual(15)
      expect(Character.getType(12)).toEqual(15)
      expect(Character.getType(13)).toEqual(15)
      expect(Character.getType(14)).toEqual(15)
      expect(Character.getType(15)).toEqual(15)
      expect(Character.getType(16)).toEqual(15)
      expect(Character.getType(17)).toEqual(15)
      expect(Character.getType(18)).toEqual(15)
      expect(Character.getType(19)).toEqual(15)
      expect(Character.getType(20)).toEqual(15)
      expect(Character.getType(21)).toEqual(15)
      expect(Character.getType(22)).toEqual(15)
      expect(Character.getType(23)).toEqual(15)
      expect(Character.getType(24)).toEqual(15)
      expect(Character.getType(25)).toEqual(15)
      expect(Character.getType(26)).toEqual(15)
      expect(Character.getType(27)).toEqual(15)
      expect(Character.getType(28)).toEqual(15)
      expect(Character.getType(29)).toEqual(15)
      expect(Character.getType(30)).toEqual(15)
      expect(Character.getType(31)).toEqual(15)
      expect(Character.getType(32)).toEqual(12)
      expect(Character.getType(33)).toEqual(24)
      expect(Character.getType(34)).toEqual(24)
      expect(Character.getType(35)).toEqual(24)
      expect(Character.getType(36)).toEqual(26)
      expect(Character.getType(37)).toEqual(24)
      expect(Character.getType(38)).toEqual(24)
      expect(Character.getType(39)).toEqual(24)
      expect(Character.getType(40)).toEqual(21)
      expect(Character.getType(41)).toEqual(22)
      expect(Character.getType(42)).toEqual(24)
      expect(Character.getType(43)).toEqual(25)
      expect(Character.getType(44)).toEqual(24)
      expect(Character.getType(45)).toEqual(20)
      expect(Character.getType(46)).toEqual(24)
      expect(Character.getType(47)).toEqual(24)
      expect(Character.getType(48)).toEqual(9)
      expect(Character.getType(49)).toEqual(9)
      expect(Character.getType(50)).toEqual(9)
      expect(Character.getType(51)).toEqual(9)
      expect(Character.getType(52)).toEqual(9)
      expect(Character.getType(53)).toEqual(9)
      expect(Character.getType(54)).toEqual(9)
      expect(Character.getType(55)).toEqual(9)
      expect(Character.getType(56)).toEqual(9)
      expect(Character.getType(57)).toEqual(9)
      expect(Character.getType(58)).toEqual(24)
      expect(Character.getType(59)).toEqual(24)
      expect(Character.getType(60)).toEqual(25)
      expect(Character.getType(61)).toEqual(25)
      expect(Character.getType(62)).toEqual(25)
      expect(Character.getType(63)).toEqual(24)
      expect(Character.getType(64)).toEqual(24)
      expect(Character.getType(65)).toEqual(1)
      expect(Character.getType(66)).toEqual(1)
      expect(Character.getType(67)).toEqual(1)
      expect(Character.getType(68)).toEqual(1)
      expect(Character.getType(69)).toEqual(1)
      expect(Character.getType(70)).toEqual(1)
      expect(Character.getType(71)).toEqual(1)
      expect(Character.getType(72)).toEqual(1)
      expect(Character.getType(73)).toEqual(1)
      expect(Character.getType(74)).toEqual(1)
      expect(Character.getType(75)).toEqual(1)
      expect(Character.getType(76)).toEqual(1)
      expect(Character.getType(77)).toEqual(1)
      expect(Character.getType(78)).toEqual(1)
      expect(Character.getType(79)).toEqual(1)
      expect(Character.getType(80)).toEqual(1)
      expect(Character.getType(81)).toEqual(1)
      expect(Character.getType(82)).toEqual(1)
      expect(Character.getType(83)).toEqual(1)
      expect(Character.getType(84)).toEqual(1)
      expect(Character.getType(85)).toEqual(1)
      expect(Character.getType(86)).toEqual(1)
      expect(Character.getType(87)).toEqual(1)
      expect(Character.getType(88)).toEqual(1)
      expect(Character.getType(89)).toEqual(1)
      expect(Character.getType(90)).toEqual(1)
      expect(Character.getType(91)).toEqual(21)
      expect(Character.getType(92)).toEqual(24)
      expect(Character.getType(93)).toEqual(22)
      expect(Character.getType(94)).toEqual(27)
      expect(Character.getType(95)).toEqual(23)
      expect(Character.getType(96)).toEqual(27)
      expect(Character.getType(97)).toEqual(2)
      expect(Character.getType(98)).toEqual(2)
      expect(Character.getType(99)).toEqual(2)
      expect(Character.getType(100)).toEqual(2)
      expect(Character.getType(101)).toEqual(2)
      expect(Character.getType(102)).toEqual(2)
      expect(Character.getType(103)).toEqual(2)
      expect(Character.getType(104)).toEqual(2)
      expect(Character.getType(105)).toEqual(2)
      expect(Character.getType(106)).toEqual(2)
      expect(Character.getType(107)).toEqual(2)
      expect(Character.getType(108)).toEqual(2)
      expect(Character.getType(109)).toEqual(2)
      expect(Character.getType(110)).toEqual(2)
      expect(Character.getType(111)).toEqual(2)
      expect(Character.getType(112)).toEqual(2)
      expect(Character.getType(113)).toEqual(2)
      expect(Character.getType(114)).toEqual(2)
      expect(Character.getType(115)).toEqual(2)
      expect(Character.getType(116)).toEqual(2)
      expect(Character.getType(117)).toEqual(2)
      expect(Character.getType(118)).toEqual(2)
      expect(Character.getType(119)).toEqual(2)
      expect(Character.getType(120)).toEqual(2)
      expect(Character.getType(121)).toEqual(2)
      expect(Character.getType(122)).toEqual(2)
      expect(Character.getType(123)).toEqual(21)
      expect(Character.getType(124)).toEqual(25)
      expect(Character.getType(125)).toEqual(22)
      expect(Character.getType(126)).toEqual(25)
      expect(Character.getType(127)).toEqual(15)
      expect(Character.getType(128)).toEqual(15)
      expect(Character.getType(129)).toEqual(15)
      expect(Character.getType(130)).toEqual(15)
      expect(Character.getType(131)).toEqual(15)
      expect(Character.getType(132)).toEqual(15)
      expect(Character.getType(133)).toEqual(15)
      expect(Character.getType(134)).toEqual(15)
      expect(Character.getType(135)).toEqual(15)
      expect(Character.getType(136)).toEqual(15)
      expect(Character.getType(137)).toEqual(15)
      expect(Character.getType(138)).toEqual(15)
      expect(Character.getType(139)).toEqual(15)
      expect(Character.getType(140)).toEqual(15)
      expect(Character.getType(141)).toEqual(15)
      expect(Character.getType(142)).toEqual(15)
      expect(Character.getType(143)).toEqual(15)
      expect(Character.getType(144)).toEqual(15)
      expect(Character.getType(145)).toEqual(15)
      expect(Character.getType(146)).toEqual(15)
      expect(Character.getType(147)).toEqual(15)
      expect(Character.getType(148)).toEqual(15)
      expect(Character.getType(149)).toEqual(15)
      expect(Character.getType(150)).toEqual(15)
      expect(Character.getType(151)).toEqual(15)
      expect(Character.getType(152)).toEqual(15)
      expect(Character.getType(153)).toEqual(15)
      expect(Character.getType(154)).toEqual(15)
      expect(Character.getType(155)).toEqual(15)
      expect(Character.getType(156)).toEqual(15)
      expect(Character.getType(157)).toEqual(15)
      expect(Character.getType(158)).toEqual(15)
      expect(Character.getType(159)).toEqual(15)
      expect(Character.getType(160)).toEqual(12)
      expect(Character.getType(161)).toEqual(24)
      expect(Character.getType(162)).toEqual(26)
      expect(Character.getType(163)).toEqual(26)
      expect(Character.getType(164)).toEqual(26)
      expect(Character.getType(165)).toEqual(26)
      expect(Character.getType(166)).toEqual(28)
      expect(Character.getType(167)).toEqual(24)
      expect(Character.getType(168)).toEqual(27)
      expect(Character.getType(169)).toEqual(28)
      expect(Character.getType(170)).toEqual(5)
      expect(Character.getType(171)).toEqual(29)
      expect(Character.getType(172)).toEqual(25)
      expect(Character.getType(173)).toEqual(16)
      expect(Character.getType(174)).toEqual(28)
      expect(Character.getType(175)).toEqual(27)
      expect(Character.getType(176)).toEqual(28)
      expect(Character.getType(177)).toEqual(25)
      expect(Character.getType(178)).toEqual(11)
      expect(Character.getType(179)).toEqual(11)
      expect(Character.getType(180)).toEqual(27)
      expect(Character.getType(181)).toEqual(2)
      expect(Character.getType(182)).toEqual(24)
      expect(Character.getType(183)).toEqual(24)
      expect(Character.getType(184)).toEqual(27)
      expect(Character.getType(185)).toEqual(11)
      expect(Character.getType(186)).toEqual(5)
      expect(Character.getType(187)).toEqual(30)
      expect(Character.getType(188)).toEqual(11)
      expect(Character.getType(189)).toEqual(11)
      expect(Character.getType(190)).toEqual(11)
      expect(Character.getType(191)).toEqual(24)
      expect(Character.getType(192)).toEqual(1)
      expect(Character.getType(193)).toEqual(1)
      expect(Character.getType(194)).toEqual(1)
      expect(Character.getType(195)).toEqual(1)
      expect(Character.getType(196)).toEqual(1)
      expect(Character.getType(197)).toEqual(1)
      expect(Character.getType(198)).toEqual(1)
      expect(Character.getType(199)).toEqual(1)
      expect(Character.getType(200)).toEqual(1)
      expect(Character.getType(201)).toEqual(1)
      expect(Character.getType(202)).toEqual(1)
      expect(Character.getType(203)).toEqual(1)
      expect(Character.getType(204)).toEqual(1)
      expect(Character.getType(205)).toEqual(1)
      expect(Character.getType(206)).toEqual(1)
      expect(Character.getType(207)).toEqual(1)
      expect(Character.getType(208)).toEqual(1)
      expect(Character.getType(209)).toEqual(1)
      expect(Character.getType(210)).toEqual(1)
      expect(Character.getType(211)).toEqual(1)
      expect(Character.getType(212)).toEqual(1)
      expect(Character.getType(213)).toEqual(1)
      expect(Character.getType(214)).toEqual(1)
      expect(Character.getType(215)).toEqual(25)
      expect(Character.getType(216)).toEqual(1)
      expect(Character.getType(217)).toEqual(1)
      expect(Character.getType(218)).toEqual(1)
      expect(Character.getType(219)).toEqual(1)
      expect(Character.getType(220)).toEqual(1)
      expect(Character.getType(221)).toEqual(1)
      expect(Character.getType(222)).toEqual(1)
      expect(Character.getType(223)).toEqual(2)
      expect(Character.getType(224)).toEqual(2)
      expect(Character.getType(225)).toEqual(2)
      expect(Character.getType(226)).toEqual(2)
      expect(Character.getType(227)).toEqual(2)
      expect(Character.getType(228)).toEqual(2)
      expect(Character.getType(229)).toEqual(2)
      expect(Character.getType(230)).toEqual(2)
      expect(Character.getType(231)).toEqual(2)
      expect(Character.getType(232)).toEqual(2)
      expect(Character.getType(233)).toEqual(2)
      expect(Character.getType(234)).toEqual(2)
      expect(Character.getType(235)).toEqual(2)
      expect(Character.getType(236)).toEqual(2)
      expect(Character.getType(237)).toEqual(2)
      expect(Character.getType(238)).toEqual(2)
      expect(Character.getType(239)).toEqual(2)
      expect(Character.getType(240)).toEqual(2)
      expect(Character.getType(241)).toEqual(2)
      expect(Character.getType(242)).toEqual(2)
      expect(Character.getType(243)).toEqual(2)
      expect(Character.getType(244)).toEqual(2)
      expect(Character.getType(245)).toEqual(2)
      expect(Character.getType(246)).toEqual(2)
      expect(Character.getType(247)).toEqual(25)
      expect(Character.getType(248)).toEqual(2)
      expect(Character.getType(249)).toEqual(2)
      expect(Character.getType(250)).toEqual(2)
      expect(Character.getType(251)).toEqual(2)
      expect(Character.getType(252)).toEqual(2)
      expect(Character.getType(253)).toEqual(2)
      expect(Character.getType(254)).toEqual(2)
      expect(Character.getType(255)).toEqual(2)
      expect(Character.getType(256)).toEqual(1)
      expect(Character.getType(257)).toEqual(2)
      expect(Character.getType(258)).toEqual(1)
      expect(Character.getType(259)).toEqual(2)
      expect(Character.getType(260)).toEqual(1)
      expect(Character.getType(261)).toEqual(2)
      expect(Character.getType(262)).toEqual(1)
      expect(Character.getType(263)).toEqual(2)
      expect(Character.getType(264)).toEqual(1)
      expect(Character.getType(265)).toEqual(2)
      expect(Character.getType(266)).toEqual(1)
      expect(Character.getType(267)).toEqual(2)
      expect(Character.getType(268)).toEqual(1)
      expect(Character.getType(269)).toEqual(2)
      expect(Character.getType(270)).toEqual(1)
      expect(Character.getType(271)).toEqual(2)
      expect(Character.getType(272)).toEqual(1)
      expect(Character.getType(273)).toEqual(2)
      expect(Character.getType(274)).toEqual(1)
      expect(Character.getType(275)).toEqual(2)
      expect(Character.getType(276)).toEqual(1)
      expect(Character.getType(277)).toEqual(2)
      expect(Character.getType(278)).toEqual(1)
      expect(Character.getType(279)).toEqual(2)
      expect(Character.getType(280)).toEqual(1)
      expect(Character.getType(281)).toEqual(2)
      expect(Character.getType(282)).toEqual(1)
      expect(Character.getType(283)).toEqual(2)
      expect(Character.getType(284)).toEqual(1)
      expect(Character.getType(285)).toEqual(2)
      expect(Character.getType(286)).toEqual(1)
      expect(Character.getType(287)).toEqual(2)
      expect(Character.getType(288)).toEqual(1)
      expect(Character.getType(289)).toEqual(2)
      expect(Character.getType(290)).toEqual(1)
      expect(Character.getType(291)).toEqual(2)
      expect(Character.getType(292)).toEqual(1)
      expect(Character.getType(293)).toEqual(2)
      expect(Character.getType(294)).toEqual(1)
      expect(Character.getType(295)).toEqual(2)
      expect(Character.getType(296)).toEqual(1)
      expect(Character.getType(297)).toEqual(2)
      expect(Character.getType(298)).toEqual(1)
      expect(Character.getType(299)).toEqual(2)

      // 400 randomly chosen characters
      expect(Character.getType(2581)).toEqual(5)
      expect(Character.getType(9237)).toEqual(28)
      expect(Character.getType(10583)).toEqual(25)
      expect(Character.getType(14526)).toEqual(5)
      expect(Character.getType(16468)).toEqual(5)
      expect(Character.getType(18538)).toEqual(5)
      expect(Character.getType(21037)).toEqual(5)
      expect(Character.getType(21880)).toEqual(5)
      expect(Character.getType(26084)).toEqual(5)
      expect(Character.getType(29876)).toEqual(5)
      expect(Character.getType(31553)).toEqual(5)
      expect(Character.getType(33192)).toEqual(5)
      expect(Character.getType(34508)).toEqual(5)
      expect(Character.getType(38994)).toEqual(5)
      expect(Character.getType(41384)).toEqual(5)
      expect(Character.getType(41661)).toEqual(5)
      expect(Character.getType(43349)).toEqual(0)
      expect(Character.getType(45859)).toEqual(5)
      expect(Character.getType(46867)).toEqual(5)
      expect(Character.getType(48796)).toEqual(5)
      expect(Character.getType(50755)).toEqual(5)
      expect(Character.getType(54077)).toEqual(5)
      expect(Character.getType(55534)).toEqual(19)
      expect(Character.getType(57625)).toEqual(18)
      expect(Character.getType(59186)).toEqual(18)
      expect(Character.getType(64622)).toEqual(5)
      expect(Character.getType(67177)).toEqual(0)
      expect(Character.getType(73067)).toEqual(0)
      expect(Character.getType(78228)).toEqual(5)
      expect(Character.getType(80656)).toEqual(0)
      expect(Character.getType(81498)).toEqual(0)
      expect(Character.getType(84617)).toEqual(0)
      expect(Character.getType(86083)).toEqual(0)
      expect(Character.getType(87255)).toEqual(0)
      expect(Character.getType(88238)).toEqual(0)
      expect(Character.getType(93812)).toEqual(0)
      expect(Character.getType(95123)).toEqual(0)
      expect(Character.getType(95326)).toEqual(0)
      expect(Character.getType(96045)).toEqual(0)
      expect(Character.getType(100120)).toEqual(0)
      expect(Character.getType(103365)).toEqual(0)
      expect(Character.getType(103707)).toEqual(0)
      expect(Character.getType(104477)).toEqual(0)
      expect(Character.getType(106587)).toEqual(0)
      expect(Character.getType(115218)).toEqual(0)
      expect(Character.getType(115801)).toEqual(0)
      expect(Character.getType(118758)).toEqual(0)
      expect(Character.getType(120024)).toEqual(1)
      expect(Character.getType(121054)).toEqual(0)
      expect(Character.getType(123903)).toEqual(0)
      expect(Character.getType(126770)).toEqual(0)
      expect(Character.getType(130632)).toEqual(0)
      expect(Character.getType(131952)).toEqual(5)
      expect(Character.getType(133129)).toEqual(5)
      expect(Character.getType(135815)).toEqual(5)
      expect(Character.getType(136969)).toEqual(5)
      expect(Character.getType(138445)).toEqual(5)
      expect(Character.getType(147888)).toEqual(5)
      expect(Character.getType(152546)).toEqual(5)
      expect(Character.getType(158177)).toEqual(5)
      expect(Character.getType(163037)).toEqual(5)
      expect(Character.getType(165823)).toEqual(5)
      expect(Character.getType(169103)).toEqual(5)
      expect(Character.getType(171304)).toEqual(5)
      expect(Character.getType(172648)).toEqual(5)
      expect(Character.getType(178250)).toEqual(0)
      expect(Character.getType(180704)).toEqual(0)
      expect(Character.getType(181598)).toEqual(0)
      expect(Character.getType(186964)).toEqual(0)
      expect(Character.getType(189826)).toEqual(0)
      expect(Character.getType(198340)).toEqual(0)
      expect(Character.getType(198519)).toEqual(0)
      expect(Character.getType(198994)).toEqual(0)
      expect(Character.getType(205472)).toEqual(0)
      expect(Character.getType(207585)).toEqual(0)
      expect(Character.getType(207958)).toEqual(0)
      expect(Character.getType(208721)).toEqual(0)
      expect(Character.getType(219582)).toEqual(0)
      expect(Character.getType(220968)).toEqual(0)
      expect(Character.getType(227170)).toEqual(0)
      expect(Character.getType(227743)).toEqual(0)
      expect(Character.getType(228515)).toEqual(0)
      expect(Character.getType(229394)).toEqual(0)
      expect(Character.getType(231801)).toEqual(0)
      expect(Character.getType(232551)).toEqual(0)
      expect(Character.getType(236092)).toEqual(0)
      expect(Character.getType(238859)).toEqual(0)
      expect(Character.getType(252799)).toEqual(0)
      expect(Character.getType(255563)).toEqual(0)
      expect(Character.getType(255895)).toEqual(0)
      expect(Character.getType(259241)).toEqual(0)
      expect(Character.getType(262761)).toEqual(0)
      expect(Character.getType(267258)).toEqual(0)
      expect(Character.getType(269354)).toEqual(0)
      expect(Character.getType(269509)).toEqual(0)
      expect(Character.getType(270726)).toEqual(0)
      expect(Character.getType(271384)).toEqual(0)
      expect(Character.getType(273382)).toEqual(0)
      expect(Character.getType(276703)).toEqual(0)
      expect(Character.getType(283121)).toEqual(0)
      expect(Character.getType(291093)).toEqual(0)
      expect(Character.getType(291551)).toEqual(0)
      expect(Character.getType(293904)).toEqual(0)
      expect(Character.getType(298484)).toEqual(0)
      expect(Character.getType(301055)).toEqual(0)
      expect(Character.getType(301379)).toEqual(0)
      expect(Character.getType(304943)).toEqual(0)
      expect(Character.getType(308860)).toEqual(0)
      expect(Character.getType(309265)).toEqual(0)
      expect(Character.getType(309466)).toEqual(0)
      expect(Character.getType(317057)).toEqual(0)
      expect(Character.getType(317814)).toEqual(0)
      expect(Character.getType(317888)).toEqual(0)
      expect(Character.getType(318020)).toEqual(0)
      expect(Character.getType(319842)).toEqual(0)
      expect(Character.getType(319936)).toEqual(0)
      expect(Character.getType(323455)).toEqual(0)
      expect(Character.getType(325009)).toEqual(0)
      expect(Character.getType(326679)).toEqual(0)
      expect(Character.getType(334555)).toEqual(0)
      expect(Character.getType(334667)).toEqual(0)
      expect(Character.getType(338205)).toEqual(0)
      expect(Character.getType(344885)).toEqual(0)
      expect(Character.getType(346752)).toEqual(0)
      expect(Character.getType(353260)).toEqual(0)
      expect(Character.getType(353840)).toEqual(0)
      expect(Character.getType(360041)).toEqual(0)
      expect(Character.getType(372920)).toEqual(0)
      expect(Character.getType(373103)).toEqual(0)
      expect(Character.getType(379356)).toEqual(0)
      expect(Character.getType(379837)).toEqual(0)
      expect(Character.getType(381734)).toEqual(0)
      expect(Character.getType(383920)).toEqual(0)
      expect(Character.getType(386450)).toEqual(0)
      expect(Character.getType(393443)).toEqual(0)
      expect(Character.getType(396435)).toEqual(0)
      expect(Character.getType(400158)).toEqual(0)
      expect(Character.getType(405132)).toEqual(0)
      expect(Character.getType(406139)).toEqual(0)
      expect(Character.getType(406817)).toEqual(0)
      expect(Character.getType(410764)).toEqual(0)
      expect(Character.getType(414453)).toEqual(0)
      expect(Character.getType(419295)).toEqual(0)
      expect(Character.getType(419492)).toEqual(0)
      expect(Character.getType(420616)).toEqual(0)
      expect(Character.getType(422854)).toEqual(0)
      expect(Character.getType(423357)).toEqual(0)
      expect(Character.getType(423620)).toEqual(0)
      expect(Character.getType(423847)).toEqual(0)
      expect(Character.getType(428863)).toEqual(0)
      expect(Character.getType(429006)).toEqual(0)
      expect(Character.getType(431107)).toEqual(0)
      expect(Character.getType(432114)).toEqual(0)
      expect(Character.getType(433251)).toEqual(0)
      expect(Character.getType(435139)).toEqual(0)
      expect(Character.getType(436750)).toEqual(0)
      expect(Character.getType(442304)).toEqual(0)
      expect(Character.getType(445467)).toEqual(0)
      expect(Character.getType(449619)).toEqual(0)
      expect(Character.getType(451649)).toEqual(0)
      expect(Character.getType(452796)).toEqual(0)
      expect(Character.getType(459845)).toEqual(0)
      expect(Character.getType(462233)).toEqual(0)
      expect(Character.getType(463465)).toEqual(0)
      expect(Character.getType(465403)).toEqual(0)
      expect(Character.getType(471112)).toEqual(0)
      expect(Character.getType(471280)).toEqual(0)
      expect(Character.getType(471634)).toEqual(0)
      expect(Character.getType(473755)).toEqual(0)
      expect(Character.getType(478248)).toEqual(0)
      expect(Character.getType(480678)).toEqual(0)
      expect(Character.getType(481366)).toEqual(0)
      expect(Character.getType(484780)).toEqual(0)
      expect(Character.getType(485524)).toEqual(0)
      expect(Character.getType(486494)).toEqual(0)
      expect(Character.getType(487220)).toEqual(0)
      expect(Character.getType(489888)).toEqual(0)
      expect(Character.getType(496358)).toEqual(0)
      expect(Character.getType(497004)).toEqual(0)
      expect(Character.getType(501635)).toEqual(0)
      expect(Character.getType(501664)).toEqual(0)
      expect(Character.getType(502672)).toEqual(0)
      expect(Character.getType(504847)).toEqual(0)
      expect(Character.getType(505791)).toEqual(0)
      expect(Character.getType(506017)).toEqual(0)
      expect(Character.getType(507855)).toEqual(0)
      expect(Character.getType(509653)).toEqual(0)
      expect(Character.getType(510983)).toEqual(0)
      expect(Character.getType(514728)).toEqual(0)
      expect(Character.getType(515825)).toEqual(0)
      expect(Character.getType(516177)).toEqual(0)
      expect(Character.getType(517858)).toEqual(0)
      expect(Character.getType(521649)).toEqual(0)
      expect(Character.getType(524928)).toEqual(0)
      expect(Character.getType(525367)).toEqual(0)
      expect(Character.getType(529235)).toEqual(0)
      expect(Character.getType(530004)).toEqual(0)
      expect(Character.getType(531799)).toEqual(0)
      expect(Character.getType(534476)).toEqual(0)
      expect(Character.getType(536386)).toEqual(0)
      expect(Character.getType(536507)).toEqual(0)
      expect(Character.getType(537192)).toEqual(0)
      expect(Character.getType(541286)).toEqual(0)
      expect(Character.getType(546959)).toEqual(0)
      expect(Character.getType(554955)).toEqual(0)
      expect(Character.getType(556317)).toEqual(0)
      expect(Character.getType(562153)).toEqual(0)
      expect(Character.getType(563963)).toEqual(0)
      expect(Character.getType(564020)).toEqual(0)
      expect(Character.getType(570471)).toEqual(0)
      expect(Character.getType(574309)).toEqual(0)
      expect(Character.getType(574487)).toEqual(0)
      expect(Character.getType(574547)).toEqual(0)
      expect(Character.getType(579326)).toEqual(0)
      expect(Character.getType(579428)).toEqual(0)
      expect(Character.getType(580567)).toEqual(0)
      expect(Character.getType(588479)).toEqual(0)
      expect(Character.getType(591039)).toEqual(0)
      expect(Character.getType(593320)).toEqual(0)
      expect(Character.getType(595035)).toEqual(0)
      expect(Character.getType(596829)).toEqual(0)
      expect(Character.getType(597596)).toEqual(0)
      expect(Character.getType(600146)).toEqual(0)
      expect(Character.getType(601789)).toEqual(0)
      expect(Character.getType(603308)).toEqual(0)
      expect(Character.getType(608296)).toEqual(0)
      expect(Character.getType(612581)).toEqual(0)
      expect(Character.getType(624001)).toEqual(0)
      expect(Character.getType(626774)).toEqual(0)
      expect(Character.getType(629729)).toEqual(0)
      expect(Character.getType(635306)).toEqual(0)
      expect(Character.getType(635481)).toEqual(0)
      expect(Character.getType(636738)).toEqual(0)
      expect(Character.getType(638187)).toEqual(0)
      expect(Character.getType(639000)).toEqual(0)
      expect(Character.getType(639171)).toEqual(0)
      expect(Character.getType(639770)).toEqual(0)
      expect(Character.getType(641295)).toEqual(0)
      expect(Character.getType(641459)).toEqual(0)
      expect(Character.getType(646699)).toEqual(0)
      expect(Character.getType(653350)).toEqual(0)
      expect(Character.getType(654242)).toEqual(0)
      expect(Character.getType(658888)).toEqual(0)
      expect(Character.getType(658937)).toEqual(0)
      expect(Character.getType(662614)).toEqual(0)
      expect(Character.getType(665575)).toEqual(0)
      expect(Character.getType(666687)).toEqual(0)
      expect(Character.getType(670456)).toEqual(0)
      expect(Character.getType(673425)).toEqual(0)
      expect(Character.getType(675808)).toEqual(0)
      expect(Character.getType(676819)).toEqual(0)
      expect(Character.getType(678369)).toEqual(0)
      expect(Character.getType(679339)).toEqual(0)
      expect(Character.getType(691315)).toEqual(0)
      expect(Character.getType(692032)).toEqual(0)
      expect(Character.getType(692992)).toEqual(0)
      expect(Character.getType(700100)).toEqual(0)
      expect(Character.getType(701010)).toEqual(0)
      expect(Character.getType(701221)).toEqual(0)
      expect(Character.getType(703413)).toEqual(0)
      expect(Character.getType(705847)).toEqual(0)
      expect(Character.getType(710671)).toEqual(0)
      expect(Character.getType(719904)).toEqual(0)
      expect(Character.getType(720047)).toEqual(0)
      expect(Character.getType(721637)).toEqual(0)
      expect(Character.getType(722513)).toEqual(0)
      expect(Character.getType(725977)).toEqual(0)
      expect(Character.getType(726721)).toEqual(0)
      expect(Character.getType(727866)).toEqual(0)
      expect(Character.getType(729810)).toEqual(0)
      expect(Character.getType(730838)).toEqual(0)
      expect(Character.getType(730973)).toEqual(0)
      expect(Character.getType(733586)).toEqual(0)
      expect(Character.getType(740086)).toEqual(0)
      expect(Character.getType(743213)).toEqual(0)
      expect(Character.getType(744503)).toEqual(0)
      expect(Character.getType(745502)).toEqual(0)
      expect(Character.getType(748676)).toEqual(0)
      expect(Character.getType(757589)).toEqual(0)
      expect(Character.getType(760894)).toEqual(0)
      expect(Character.getType(761113)).toEqual(0)
      expect(Character.getType(762701)).toEqual(0)
      expect(Character.getType(764666)).toEqual(0)
      expect(Character.getType(769454)).toEqual(0)
      expect(Character.getType(779853)).toEqual(0)
      expect(Character.getType(780781)).toEqual(0)
      expect(Character.getType(781852)).toEqual(0)
      expect(Character.getType(781978)).toEqual(0)
      expect(Character.getType(785688)).toEqual(0)
      expect(Character.getType(786900)).toEqual(0)
      expect(Character.getType(790949)).toEqual(0)
      expect(Character.getType(791871)).toEqual(0)
      expect(Character.getType(793467)).toEqual(0)
      expect(Character.getType(801002)).toEqual(0)
      expect(Character.getType(804941)).toEqual(0)
      expect(Character.getType(805357)).toEqual(0)
      expect(Character.getType(818312)).toEqual(0)
      expect(Character.getType(819974)).toEqual(0)
      expect(Character.getType(819991)).toEqual(0)
      expect(Character.getType(820603)).toEqual(0)
      expect(Character.getType(825262)).toEqual(0)
      expect(Character.getType(825983)).toEqual(0)
      expect(Character.getType(826693)).toEqual(0)
      expect(Character.getType(835921)).toEqual(0)
      expect(Character.getType(836149)).toEqual(0)
      expect(Character.getType(836435)).toEqual(0)
      expect(Character.getType(837216)).toEqual(0)
      expect(Character.getType(839648)).toEqual(0)
      expect(Character.getType(842471)).toEqual(0)
      expect(Character.getType(845710)).toEqual(0)
      expect(Character.getType(847881)).toEqual(0)
      expect(Character.getType(848209)).toEqual(0)
      expect(Character.getType(848861)).toEqual(0)
      expect(Character.getType(856409)).toEqual(0)
      expect(Character.getType(863204)).toEqual(0)
      expect(Character.getType(865730)).toEqual(0)
      expect(Character.getType(867884)).toEqual(0)
      expect(Character.getType(869447)).toEqual(0)
      expect(Character.getType(872726)).toEqual(0)
      expect(Character.getType(877063)).toEqual(0)
      expect(Character.getType(877663)).toEqual(0)
      expect(Character.getType(879020)).toEqual(0)
      expect(Character.getType(880659)).toEqual(0)
      expect(Character.getType(883941)).toEqual(0)
      expect(Character.getType(888825)).toEqual(0)
      expect(Character.getType(891451)).toEqual(0)
      expect(Character.getType(899396)).toEqual(0)
      expect(Character.getType(902996)).toEqual(0)
      expect(Character.getType(904030)).toEqual(0)
      expect(Character.getType(904638)).toEqual(0)
      expect(Character.getType(910762)).toEqual(0)
      expect(Character.getType(913805)).toEqual(0)
      expect(Character.getType(916889)).toEqual(0)
      expect(Character.getType(918899)).toEqual(0)
      expect(Character.getType(920560)).toEqual(0)
      expect(Character.getType(923643)).toEqual(0)
      expect(Character.getType(927674)).toEqual(0)
      expect(Character.getType(941501)).toEqual(0)
      expect(Character.getType(942293)).toEqual(0)
      expect(Character.getType(942398)).toEqual(0)
      expect(Character.getType(947312)).toEqual(0)
      expect(Character.getType(947843)).toEqual(0)
      expect(Character.getType(954629)).toEqual(0)
      expect(Character.getType(957954)).toEqual(0)
      expect(Character.getType(957997)).toEqual(0)
      expect(Character.getType(958413)).toEqual(0)
      expect(Character.getType(959866)).toEqual(0)
      expect(Character.getType(960082)).toEqual(0)
      expect(Character.getType(960943)).toEqual(0)
      expect(Character.getType(966692)).toEqual(0)
      expect(Character.getType(973576)).toEqual(0)
      expect(Character.getType(975710)).toEqual(0)
      expect(Character.getType(976432)).toEqual(0)
      expect(Character.getType(979277)).toEqual(0)
      expect(Character.getType(981840)).toEqual(0)
      expect(Character.getType(981940)).toEqual(0)
      expect(Character.getType(982487)).toEqual(0)
      expect(Character.getType(985623)).toEqual(18)
      expect(Character.getType(989149)).toEqual(18)
      expect(Character.getType(997988)).toEqual(18)
      expect(Character.getType(1000132)).toEqual(18)
      expect(Character.getType(1002215)).toEqual(18)
      expect(Character.getType(1002917)).toEqual(18)
      expect(Character.getType(1004580)).toEqual(18)
      expect(Character.getType(1006595)).toEqual(18)
      expect(Character.getType(1007725)).toEqual(18)
      expect(Character.getType(1010481)).toEqual(18)
      expect(Character.getType(1016838)).toEqual(18)
      expect(Character.getType(1016895)).toEqual(18)
      expect(Character.getType(1018160)).toEqual(18)
      expect(Character.getType(1020126)).toEqual(18)
      expect(Character.getType(1026576)).toEqual(18)
      expect(Character.getType(1027582)).toEqual(18)
      expect(Character.getType(1036147)).toEqual(18)
      expect(Character.getType(1036943)).toEqual(18)
      expect(Character.getType(1040859)).toEqual(18)
      expect(Character.getType(1041209)).toEqual(18)
      expect(Character.getType(1044637)).toEqual(18)
      expect(Character.getType(1045872)).toEqual(18)
      expect(Character.getType(1045977)).toEqual(18)
      expect(Character.getType(1052466)).toEqual(18)
      expect(Character.getType(1052541)).toEqual(18)
      expect(Character.getType(1052557)).toEqual(18)
      expect(Character.getType(1052647)).toEqual(18)
      expect(Character.getType(1060162)).toEqual(18)
      expect(Character.getType(1065690)).toEqual(18)
      expect(Character.getType(1078626)).toEqual(18)
      expect(Character.getType(1080764)).toEqual(18)
      expect(Character.getType(1080771)).toEqual(18)
      expect(Character.getType(1082986)).toEqual(18)
      expect(Character.getType(1083091)).toEqual(18)
      expect(Character.getType(1086585)).toEqual(18)
      expect(Character.getType(1087471)).toEqual(18)
      expect(Character.getType(1094259)).toEqual(18)
      expect(Character.getType(1097626)).toEqual(18)
      expect(Character.getType(1101125)).toEqual(18)
      expect(Character.getType(1103470)).toEqual(18)
      expect(Character.getType(1106648)).toEqual(18)
      expect(Character.getType(1107521)).toEqual(18)
      expect(Character.getType(1112476)).toEqual(18)
    }

    it("should provide isJavaLetter") {
      // 50 randomly chosen characters that produce true
      expect(Character.isJavaLetter('\u1115')).toBeTruthy
      expect(Character.isJavaLetter('\u1895')).toBeTruthy
      expect(Character.isJavaLetter('\u1E89')).toBeTruthy
      expect(Character.isJavaLetter('\u1ECF')).toBeTruthy
      expect(Character.isJavaLetter('\u3C72')).toBeTruthy
      expect(Character.isJavaLetter('\u3D16')).toBeTruthy
      expect(Character.isJavaLetter('\u426E')).toBeTruthy
      expect(Character.isJavaLetter('\u436D')).toBeTruthy
      expect(Character.isJavaLetter('\u4723')).toBeTruthy
      expect(Character.isJavaLetter('\u500F')).toBeTruthy
      expect(Character.isJavaLetter('\u52D3')).toBeTruthy
      expect(Character.isJavaLetter('\u55DA')).toBeTruthy
      expect(Character.isJavaLetter('\u571D')).toBeTruthy
      expect(Character.isJavaLetter('\u5CE6')).toBeTruthy
      expect(Character.isJavaLetter('\u5F2A')).toBeTruthy
      expect(Character.isJavaLetter('\u5F3B')).toBeTruthy
      expect(Character.isJavaLetter('\u60ED')).toBeTruthy
      expect(Character.isJavaLetter('\u611A')).toBeTruthy
      expect(Character.isJavaLetter('\u626A')).toBeTruthy
      expect(Character.isJavaLetter('\u6539')).toBeTruthy
      expect(Character.isJavaLetter('\u6609')).toBeTruthy
      expect(Character.isJavaLetter('\u0698')).toBeTruthy
      expect(Character.isJavaLetter('\u6FF6')).toBeTruthy
      expect(Character.isJavaLetter('\u7332')).toBeTruthy
      expect(Character.isJavaLetter('\u7347')).toBeTruthy
      expect(Character.isJavaLetter('\u7431')).toBeTruthy
      expect(Character.isJavaLetter('\u7F68')).toBeTruthy
      expect(Character.isJavaLetter('\u7FB9')).toBeTruthy
      expect(Character.isJavaLetter('\u7FC0')).toBeTruthy
      expect(Character.isJavaLetter('\u8603')).toBeTruthy
      expect(Character.isJavaLetter('\u8707')).toBeTruthy
      expect(Character.isJavaLetter('\u8871')).toBeTruthy
      expect(Character.isJavaLetter('\u8B1B')).toBeTruthy
      expect(Character.isJavaLetter('\u8E1B')).toBeTruthy
      expect(Character.isJavaLetter('\u9176')).toBeTruthy
      expect(Character.isJavaLetter('\u99D5')).toBeTruthy
      expect(Character.isJavaLetter('\u9D5E')).toBeTruthy
      expect(Character.isJavaLetter('\u9D62')).toBeTruthy
      expect(Character.isJavaLetter('\uA0E2')).toBeTruthy
      expect(Character.isJavaLetter('\uA61E')).toBeTruthy
      expect(Character.isJavaLetter('\uA6C3')).toBeTruthy
      expect(Character.isJavaLetter('\uAFAD')).toBeTruthy
      expect(Character.isJavaLetter('\uBB42')).toBeTruthy
      expect(Character.isJavaLetter('\uC4B5')).toBeTruthy
      expect(Character.isJavaLetter('\uC4DA')).toBeTruthy
      expect(Character.isJavaLetter('\uD301')).toBeTruthy
      expect(Character.isJavaLetter('\uD3E5')).toBeTruthy
      expect(Character.isJavaLetter('\u0EAB')).toBeTruthy
      expect(Character.isJavaLetter('\uF9C0')).toBeTruthy
      expect(Character.isJavaLetter('\uFD8F')).toBeTruthy

      // 50 randomly chosen characters that produce false
      expect(Character.isJavaLetter('\u19CF')).toBeFalsy
      expect(Character.isJavaLetter('\u23D7')).toBeFalsy
      expect(Character.isJavaLetter('\u26A9')).toBeFalsy
      expect(Character.isJavaLetter('\u2775')).toBeFalsy
      expect(Character.isJavaLetter('\u2832')).toBeFalsy
      expect(Character.isJavaLetter('\u2885')).toBeFalsy
      expect(Character.isJavaLetter('\u2930')).toBeFalsy
      expect(Character.isJavaLetter('\u2B8F')).toBeFalsy
      expect(Character.isJavaLetter('\u2CF4')).toBeFalsy
      expect(Character.isJavaLetter('\u2F70')).toBeFalsy
      expect(Character.isJavaLetter('\u0037')).toBeFalsy
      expect(Character.isJavaLetter('\u073F')).toBeFalsy
      expect(Character.isJavaLetter('\u0969')).toBeFalsy
      expect(Character.isJavaLetter('\u9FF3')).toBeFalsy
      expect(Character.isJavaLetter('\uA9E5')).toBeFalsy
      expect(Character.isJavaLetter('\uAAC1')).toBeFalsy
      expect(Character.isJavaLetter('\uAAF7')).toBeFalsy
      expect(Character.isJavaLetter('\u0B82')).toBeFalsy
      expect(Character.isJavaLetter('\uDA08')).toBeFalsy
      expect(Character.isJavaLetter('\uDA8D')).toBeFalsy
      expect(Character.isJavaLetter('\uDBC8')).toBeFalsy
      expect(Character.isJavaLetter('\uDBF7')).toBeFalsy
      expect(Character.isJavaLetter('\uDD3C')).toBeFalsy
      expect(Character.isJavaLetter('\uDDD1')).toBeFalsy
      expect(Character.isJavaLetter('\uDEC2')).toBeFalsy
      expect(Character.isJavaLetter('\uE068')).toBeFalsy
      expect(Character.isJavaLetter('\uE0A3')).toBeFalsy
      expect(Character.isJavaLetter('\uE210')).toBeFalsy
      expect(Character.isJavaLetter('\uE414')).toBeFalsy
      expect(Character.isJavaLetter('\u0E4C')).toBeFalsy
      expect(Character.isJavaLetter('\uE4FD')).toBeFalsy
      expect(Character.isJavaLetter('\uE5F7')).toBeFalsy
      expect(Character.isJavaLetter('\uE611')).toBeFalsy
      expect(Character.isJavaLetter('\u0E6A')).toBeFalsy
      expect(Character.isJavaLetter('\uE704')).toBeFalsy
      expect(Character.isJavaLetter('\uE71F')).toBeFalsy
      expect(Character.isJavaLetter('\uE7DB')).toBeFalsy
      expect(Character.isJavaLetter('\uEA9F')).toBeFalsy
      expect(Character.isJavaLetter('\uEAAB')).toBeFalsy
      expect(Character.isJavaLetter('\uEBF1')).toBeFalsy
      expect(Character.isJavaLetter('\uED24')).toBeFalsy
      expect(Character.isJavaLetter('\uED93')).toBeFalsy
      expect(Character.isJavaLetter('\uEDE1')).toBeFalsy
      expect(Character.isJavaLetter('\uEF40')).toBeFalsy
      expect(Character.isJavaLetter('\uF312')).toBeFalsy
      expect(Character.isJavaLetter('\uF6B9')).toBeFalsy
      expect(Character.isJavaLetter('\uF80F')).toBeFalsy
      expect(Character.isJavaLetter('\uFD3E')).toBeFalsy
      expect(Character.isJavaLetter('\u0FF9')).toBeFalsy
    }

    it("should provide isJavaLetterOrDigit") {
      // 50 randomly chosen characters that produce true
      expect(Character.isJavaLetterOrDigit('\u0B42')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u0C35')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u1DBD')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u1FE5')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u3572')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u3FBC')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u46E6')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u47C1')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u495B')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u49D3')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u4E38')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u5513')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u57CD')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u5884')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u5A74')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u5CEB')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u5D4B')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u5F0B')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u6889')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u69D0')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u6A40')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u6C9E')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u7237')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u7289')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u7313')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u7626')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u7C1C')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u7DC9')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u8093')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u8573')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u884E')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u90AE')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u94E8')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u956A')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u9912')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\u9E2A')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uACE6')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uB0E7')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uB3E5')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uB7CA')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uBD2C')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uBD3F')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uC155')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uC585')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uC7FC')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uD250')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uD378')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uD73B')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uFDB1')).toBeTruthy
      expect(Character.isJavaLetterOrDigit('\uFEE5')).toBeTruthy

      // 50 randomly chosen characters that produce false
      expect(Character.isJavaLetterOrDigit('\u05FD')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u08B3')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u09FE')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u0AD6')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u0F3D')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u0FCA')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u17F5')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u1A5F')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u1F4E')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u204C')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u224A')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2296')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u241C')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u247C')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2490')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u24B8')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u253B')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2755')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2A18')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2A64')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2A91')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\u2F0F')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uA4C1')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uA4C3')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uA797')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uA7C3')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uDA09')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uDB3E')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uDE30')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uDFB4')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE116')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE20D')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE21D')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE341')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE4DC')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE6F7')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE6FB')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uE785')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uEA37')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uEB1C')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uEE47')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uEEA6')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF27F')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF2FE')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF33F')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF3B1')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF5AD')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF6B1')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uF84B')).toBeFalsy
      expect(Character.isJavaLetterOrDigit('\uFF03')).toBeFalsy
    }

    it("should provide isJavaIdentifierStart") {
      // 50 randomly chosen characters that produce true
      expect(Character.isJavaIdentifierStart('\u186C')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u1F75')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u347C')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u485F')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u54A6')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u58CF')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u65F6')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u6E34')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u7404')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u77BD')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u7977')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u83A9')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u844E')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u9D7D')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\u9E7F')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uA734')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uAA81')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uACED')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uBD6B')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uBD89')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uC5E4')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uC7C7')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uD3C8')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uF972')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uFA9F')).toBeTruthy
      expect(Character.isJavaIdentifierStart('\uFDA6')).toBeTruthy
      expect(Character.isJavaIdentifierStart(132773)).toBeTruthy
      expect(Character.isJavaIdentifierStart(136707)).toBeTruthy
      expect(Character.isJavaIdentifierStart(140254)).toBeTruthy
      expect(Character.isJavaIdentifierStart(141336)).toBeTruthy
      expect(Character.isJavaIdentifierStart(147781)).toBeTruthy
      expect(Character.isJavaIdentifierStart(148525)).toBeTruthy
      expect(Character.isJavaIdentifierStart(149236)).toBeTruthy
      expect(Character.isJavaIdentifierStart(151919)).toBeTruthy
      expect(Character.isJavaIdentifierStart(153364)).toBeTruthy
      expect(Character.isJavaIdentifierStart(153898)).toBeTruthy
      expect(Character.isJavaIdentifierStart(157645)).toBeTruthy
      expect(Character.isJavaIdentifierStart(158882)).toBeTruthy
      expect(Character.isJavaIdentifierStart(161627)).toBeTruthy
      expect(Character.isJavaIdentifierStart(164324)).toBeTruthy
      expect(Character.isJavaIdentifierStart(168371)).toBeTruthy
      expect(Character.isJavaIdentifierStart(169573)).toBeTruthy
      expect(Character.isJavaIdentifierStart(171871)).toBeTruthy
      expect(Character.isJavaIdentifierStart(172017)).toBeTruthy
      expect(Character.isJavaIdentifierStart(172787)).toBeTruthy
      expect(Character.isJavaIdentifierStart(173670)).toBeTruthy
      expect(Character.isJavaIdentifierStart(175824)).toBeTruthy
      expect(Character.isJavaIdentifierStart(74462)).toBeTruthy
      expect(Character.isJavaIdentifierStart(78380)).toBeTruthy

      // 50 randomly chosen characters that produce false
      expect(Character.isJavaIdentifierStart('\uE8A1')).toBeFalsy
      expect(Character.isJavaIdentifierStart('\uEBB6')).toBeFalsy
      expect(Character.isJavaIdentifierStart(1006625)).toBeFalsy
      expect(Character.isJavaIdentifierStart(1038096)).toBeFalsy
      expect(Character.isJavaIdentifierStart(1080963)).toBeFalsy
      expect(Character.isJavaIdentifierStart(1098681)).toBeFalsy
      expect(Character.isJavaIdentifierStart(1101181)).toBeFalsy
      expect(Character.isJavaIdentifierStart(185775)).toBeFalsy
      expect(Character.isJavaIdentifierStart(200078)).toBeFalsy
      expect(Character.isJavaIdentifierStart(212082)).toBeFalsy
      expect(Character.isJavaIdentifierStart(219401)).toBeFalsy
      expect(Character.isJavaIdentifierStart(223687)).toBeFalsy
      expect(Character.isJavaIdentifierStart(230929)).toBeFalsy
      expect(Character.isJavaIdentifierStart(238776)).toBeFalsy
      expect(Character.isJavaIdentifierStart(275525)).toBeFalsy
      expect(Character.isJavaIdentifierStart(297540)).toBeFalsy
      expect(Character.isJavaIdentifierStart(310684)).toBeFalsy
      expect(Character.isJavaIdentifierStart(323571)).toBeFalsy
      expect(Character.isJavaIdentifierStart(340895)).toBeFalsy
      expect(Character.isJavaIdentifierStart(355972)).toBeFalsy
      expect(Character.isJavaIdentifierStart(382586)).toBeFalsy
      expect(Character.isJavaIdentifierStart(385597)).toBeFalsy
      expect(Character.isJavaIdentifierStart(438811)).toBeFalsy
      expect(Character.isJavaIdentifierStart(442235)).toBeFalsy
      expect(Character.isJavaIdentifierStart(446274)).toBeFalsy
      expect(Character.isJavaIdentifierStart(454075)).toBeFalsy
      expect(Character.isJavaIdentifierStart(464326)).toBeFalsy
      expect(Character.isJavaIdentifierStart(469494)).toBeFalsy
      expect(Character.isJavaIdentifierStart(473583)).toBeFalsy
      expect(Character.isJavaIdentifierStart(481617)).toBeFalsy
      expect(Character.isJavaIdentifierStart(492310)).toBeFalsy
      expect(Character.isJavaIdentifierStart(502865)).toBeFalsy
      expect(Character.isJavaIdentifierStart(505834)).toBeFalsy
      expect(Character.isJavaIdentifierStart(521762)).toBeFalsy
      expect(Character.isJavaIdentifierStart(531102)).toBeFalsy
      expect(Character.isJavaIdentifierStart(531557)).toBeFalsy
      expect(Character.isJavaIdentifierStart(573907)).toBeFalsy
      expect(Character.isJavaIdentifierStart(590651)).toBeFalsy
      expect(Character.isJavaIdentifierStart(602729)).toBeFalsy
      expect(Character.isJavaIdentifierStart(607107)).toBeFalsy
      expect(Character.isJavaIdentifierStart(657499)).toBeFalsy
      expect(Character.isJavaIdentifierStart(684199)).toBeFalsy
      expect(Character.isJavaIdentifierStart(727324)).toBeFalsy
      expect(Character.isJavaIdentifierStart(757790)).toBeFalsy
      expect(Character.isJavaIdentifierStart(783173)).toBeFalsy
      expect(Character.isJavaIdentifierStart(805578)).toBeFalsy
      expect(Character.isJavaIdentifierStart(834136)).toBeFalsy
      expect(Character.isJavaIdentifierStart(875116)).toBeFalsy
      expect(Character.isJavaIdentifierStart(955913)).toBeFalsy
      expect(Character.isJavaIdentifierStart(958246)).toBeFalsy
    }
  }
}
